package diffusioncalculator;

import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.geom.Rectangle2D;

import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JPanel;

import org.jfree.chart.ChartFactory;
import org.jfree.chart.ChartMouseEvent;
import org.jfree.chart.ChartMouseListener;
import org.jfree.chart.ChartPanel;
import org.jfree.chart.JFreeChart;
import org.jfree.chart.axis.ValueAxis;
import org.jfree.chart.panel.CrosshairOverlay;
import org.jfree.chart.plot.Crosshair;
import org.jfree.chart.plot.PlotOrientation;
import org.jfree.chart.plot.XYPlot;
import org.jfree.chart.renderer.xy.XYLineAndShapeRenderer;
import org.jfree.data.general.DatasetUtilities;
import org.jfree.data.xy.XYDataset;
import org.jfree.ui.RectangleEdge;

@SuppressWarnings("serial")
public class CurrentChartView extends JFrame implements ChartMouseListener {

	// Declaring the chart and panel
	JFreeChart chart;
	ChartPanel chartPanel;
	JPanel buttonPanel;
	Crosshair xCrosshair;
	Crosshair yCrosshair;
	CrosshairOverlay crosshairOverlay;
	JButton lowPassButton;
	CurrentChartModel currentModel;

	// Constructor creating the frame
	public CurrentChartView(final String title, CurrentChartModel inputCurrentModel) {

		//super(title);
		//setLayout(null);
		currentModel = inputCurrentModel;
		
		chart = createChart(currentModel.dataset);
		chartPanel = new ChartPanel(chart);
		chartPanel.setLayout(null);
		
		buttonPanel = new JPanel();
		buttonPanel.setLayout(null);
		
		this.chartPanel.addChartMouseListener(this);
		crosshairOverlay = new CrosshairOverlay();
		this.xCrosshair = new Crosshair(Double.NaN, Color.GRAY, new BasicStroke(0f));
        this.xCrosshair.setLabelVisible(true);
        this.yCrosshair = new Crosshair(Double.NaN, Color.GRAY, new BasicStroke(0f));
        this.yCrosshair.setLabelVisible(true);
        crosshairOverlay.addDomainCrosshair(xCrosshair);
        crosshairOverlay.addRangeCrosshair(yCrosshair);
        chartPanel.addOverlay(crosshairOverlay);
		
		
		chartPanel.setBounds(0, 0,(int) (Util.XSIZE*0.95), (int) (Util.YSIZE*0.8));
		add(chartPanel);
		
		lowPassButton();
		lowPassButton.setBounds(0, (int) (Util.YSIZE*0.8), 220, 25);
		buttonPanel.add(lowPassButton);
		buttonPanel.setBounds(0, (int) (Util.YSIZE*0.8),(int) (Util.XSIZE*0.95), (int) (Util.YSIZE*0.2));
		add(buttonPanel);
		
		
		setSize(Util.XSIZE, (int) (Util.YSIZE*0.9));

		setVisible(true);

	}

	// Creating the chart
	private JFreeChart createChart(final XYDataset dataset) {

		// Initializing the chart
		final JFreeChart chart = ChartFactory.createXYLineChart(
				"Current/Time Plot", // chart title
				"Time (s)", // x axis label
				"Current (A)", // y axis label
				dataset, // data
				PlotOrientation.VERTICAL, true, // include legend
				true, // tooltips
				false // urls
				);

		// Customization of the chart
		chart.setBackgroundPaint(Color.white);
		

		// Make a reference to the plot for further customization
		final XYPlot plot = chart.getXYPlot();
		plot.setBackgroundPaint(Color.lightGray);
		
		plot.setDomainGridlinePaint(Color.white);
		plot.setRangeGridlinePaint(Color.white);
		
		// Rendering the line representation
		final XYLineAndShapeRenderer renderer = new XYLineAndShapeRenderer();
		renderer.setSeriesLinesVisible(0, true);
		renderer.setSeriesShapesVisible(0, false);
		plot.setRenderer(renderer);

		return chart;

	}

	@Override
	public void chartMouseClicked(ChartMouseEvent event) {
		// TODO Auto-generated method stub
		
	}

	@Override
	public void chartMouseMoved(ChartMouseEvent event) {
		 Rectangle2D dataArea = this.chartPanel.getScreenDataArea();
	        JFreeChart chart = event.getChart();
	        XYPlot plot = (XYPlot) chart.getPlot();
	        ValueAxis xAxis = plot.getDomainAxis();
	        double x = xAxis.java2DToValue(event.getTrigger().getX(), dataArea, 
	                RectangleEdge.BOTTOM);
	        double y = DatasetUtilities.findYValue(plot.getDataset(), 0, x);
	        this.xCrosshair.setValue(x);
	        this.yCrosshair.setValue(y);
		
	}
	
	// Create button for low pass filter
		public void lowPassButton() {
			lowPassButton = new JButton("Calculate");

			// When low pass button is pressed
			lowPassButton.addActionListener(new ActionListener() {

				public void actionPerformed(ActionEvent e) {
					currentModel.lowPassFilter();
				}
			});
		}

}