package diffusioncalculator;

import java.util.ArrayList;
import java.util.Collections;

import javax.swing.JTextArea;
import javax.swing.table.DefaultTableModel;

/* Class performing all calculations on data read from the 
 * datafile, and saving it to texarea files, and tables.*/
public class ResultsModel {

	// Name of file read holder variable
	private String name;
	
	// Declaring ArrayLists, containing data calculations are performed on
	private ArrayList<Float> time;
	private ArrayList<Float> current;
	private ArrayList<Float> currentB;
	private ArrayList<Float> flux;
	private ArrayList<Float> tau;
	
	// Declaring columns making up data in the DefaultTableModel
	// instance declared below
	private Float[] rowNr;
	private Float[] timeColumn;
	private Float[] currentColumn;
	private Float[] currentBColumn;
	private Float[] fluxColumn;
	private Float[] tauColumn;
	private DefaultTableModel model;
	
	// Declaring variables used in calculations
	private float initialNoise;
	private float imax;
	private float jss;
	private int index;
	private float deff;
	private float dl;
	private float L;
	
	

	// Constructor initializing the time and current ArrayList
	public ResultsModel(ArrayList<Float> inputTime,
			ArrayList<Float> inputCurrent, String inputName) {
		name = inputName;
		time = inputTime;
		current = inputCurrent;
		currentB = new ArrayList<Float>();
		flux = new ArrayList<Float>();
		tau = new ArrayList<Float>();
	}

	// Make initial table data
	public void initialTableData() {

		// Creating model data
		model = new DefaultTableModel();

		// Declaring array size
		rowNr = new Float[time.size()];
		timeColumn = new Float[time.size()];
		currentColumn = new Float[current.size()];
		currentBColumn = new Float[current.size()];
		fluxColumn = new Float[current.size()];
		tauColumn = new Float[current.size()];

		// Making row numbers
		for (int i = 0; i < rowNr.length; i++) {
			rowNr[i] = (float) i;
		}

		// Converting data
		timeColumn = time.toArray(timeColumn);
		currentColumn = current.toArray(currentColumn);

		// Adding data to model
		model.addColumn("Nr.", rowNr);
		model.addColumn("Time", timeColumn);
		model.addColumn("Current", currentColumn);
		model.addColumn("Current - Background", currentBColumn);
		model.addColumn("J(t)/Jss", fluxColumn);
		model.addColumn("tDl/L^2", tauColumn);
	}

	// Calculating Dl from user's temperature input
	public void setDL(float inputTemp) {
		float temp = (float) (273 + inputTemp);
		dl = (float) (Util.DLCOFF * Math.exp(-Util.Q / (Util.R * temp)));
	}
	
	// set thickness from user input
		public void setThickness(float inputL) {
			L = inputL;
		}
	

	// Making the remaining lists and computing results
	public void buttonCalculate() {

		// Clearing and initializing the last lists
		currentB.clear();
		flux.clear();
		tau.clear();

		// Initializing a difference variable and the index variable
		double lastdiff = 1;
		index = 0;

		// Adding elements to the current list without background noise
		for (int i = 0; i < current.size(); i++) {
			currentB.add(current.get(i) - initialNoise);
		}

		// Finding the biggest value in the list
		imax = Collections.max(currentB);

		// Finding variable for further calculations
		jss = (imax / Util.AREAL) / Util.FARADAY;

		// Adding elements to the flux and tau lists
		for (int j = 0; j < currentB.size(); j++) {
			flux.add(((currentB.get(j) / Util.AREAL) / Util.FARADAY) / jss);
			tau.add((time.get(j) * dl) / (L*L));
			// Finding index off J(t)/Jss closest to 0.63
			if (lastdiff > Math.abs(flux.get(j) - 0.63)) {
				lastdiff = Math.abs(flux.get(j) - 0.63);
				index = j;
			}
		}
	}

	// Refresh the data in the table
	public void refreshTableData(Float inputInitialNoise) {
		initialNoise = inputInitialNoise;
		// Calculating new values
		buttonCalculate();

		// Update data
		currentBColumn = currentB.toArray(currentBColumn);
		fluxColumn = flux.toArray(fluxColumn);
		tauColumn = tau.toArray(tauColumn);

		// Deleting existing data
		model.setColumnCount(0);

		// Adding columns
		model.addColumn("Nr.", rowNr);
		model.addColumn("Time", timeColumn);
		model.addColumn("Current", currentColumn);
		model.addColumn("Current - Background", currentBColumn);
		model.addColumn("Normalized Flux", fluxColumn);
		model.addColumn("Normalized Time", tauColumn);

		// Inform model that data has been updated
		model.fireTableDataChanged();
	}

	// Refresh results data
	public void refreshResultsArea(JTextArea resultsArea) {

		// remove initial data
		resultsArea.setText(null);

		// Save tau data at index (J(t)/Jss closest to 0.63)
		float tdl = tau.get(index);

		// Computing remaining results
		float tlag = (tdl * L*L) / dl;
		deff = (L*L) / (6 * tlag);
		float comol = (float) ((imax / Util.AREAL) * L) / (Util.FARADAY * deff);
		float cog = comol * Util.HMASS;
		float coppm = (float) ((cog / Util.STEELDENSITY) * 1000000);

		// Append data to the result area
		resultsArea.append("RESULTS" + "\n" + "index" + "\t" + index + "\n"
				+ "Dl" + "\t" + dl + "\n" + "imax" + "\t" + imax +  "\n" + "tDl/L^2" + "\t" + tdl + "\n"
				+ "tlag" + "\t" + tlag + "\n" + "Deff" + "\t" + deff + "\n"
				+ "Co(mol/cm^3)" + "\t" + comol + "\n" + "Co(g/cm^3)" + "\t"
				+ cog + "\n" + "Co(ppm)" + "\t" + coppm);
	}
	
	// Declaring methods to retrieve private fields
	public float getDeff(){
		return deff;
	}
	public float getDl(){
		return dl;
	}
	public int getIndex(){
		return index;
	}
	public String getName(){
		return name;
	}
	public ArrayList<Float> getTime(){
		return time;
	}
	public ArrayList<Float> getCurrent(){
		return current;
	}
	public ArrayList<Float> getTau() {
		return tau;
	}
	public ArrayList<Float> getFlux() {
		return flux;
	}
	public ArrayList<Float> getCurrentB() {
		return currentB;
	}
	public DefaultTableModel getModel() {
		return model;
	}
	public Float getJss() {
		return jss;
	}
	
	// Declaring methods to set private fields
	public void setTime(ArrayList<Float> newTime) {
		time = newTime;
	}
}
