package diffusioncalculator;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.JTextArea;
import javax.swing.JTextField;

/* Class creating main view of the program, letting the
 * user input parameters, calculate values and plot charts.
 * Showing a scroll table with all data read from file,
 * and columns calculated from this data. */
public class ResultsView {

	// Declaring ResultsModel instance
	private ResultsModel resultsModel;

	// Declaring charts
	private PermeationChartModel permeationModel;
	private CurrentChartModel currentModel;

	// Declaring GUI components
	private JTextArea resultsArea;
	private JTextArea chartResultsArea;

	// Declaring length of specimen, initialNoise, and temperature
	// input from user
	private float L;
	private float initialNoise;
	private float temp;

	// Declare flags
	private boolean currentRun;
	private boolean fluxRun;

	// Constructor declaring and adding components to a frame.
	public ResultsView(ResultsModel inputResultsModel) {

		// Initializing a results model
		resultsModel = inputResultsModel;

		// Create and set up the window.
		final JFrame frame = new JFrame(resultsModel.getName());

		// Make the program end when the window is closed
		frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);

		// Set default layout
		frame.setLayout(null);

		// Add content
		frame.add(leftScrollTable());
		frame.add(rightPanel());

		// Set dimensions and make the frame visible
		frame.setSize((int) (Util.XSIZE * 0.7), (int) (Util.YSIZE * 0.95));
		frame.setVisible(true);

		// Declare flags
		currentRun = false;
		fluxRun = false;
	}

	// Create scroll pane with table
	public JScrollPane leftScrollTable() {

		// Create initial data input
		resultsModel.initialTableData();

		// Initialize table with data
		JTable table = new JTable(resultsModel.getModel());

		// Add scroll pane to table
		JScrollPane scrollableTable = new JScrollPane(table);
		scrollableTable
				.setHorizontalScrollBarPolicy(JScrollPane.HORIZONTAL_SCROLLBAR_ALWAYS);
		scrollableTable
				.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_ALWAYS);

		// Set boundaries and return table
		scrollableTable.setBounds(0, 0, Util.XSIZE / 2, Util.YSIZE - 70);
		return scrollableTable;
	}

	// Create right panel
	public JPanel rightPanel() {

		// Declare panel and set default layout
		JPanel rightPanel = new JPanel();
		rightPanel.setLayout(null);

		// Add text fields to the panel
		rightPanel.add(thicknessTextField());
		rightPanel.add(tempTextField());
		rightPanel.add(initialNoiseTextField());
		
		// Create initial result text area
		resultsArea = new JTextArea(10, 10);
		resultsArea.append("RESULTS" + "\n" + "index" + "\n" + "Dl" + "\n"
				+ "imax" + "\n" + "tDl/L^2" + "\n" + "tlag" + "\n" + "Deff"
				+ "\n" + "Co(mol/cm^3)" + "\n" + "Co(g/cm^3)" + "\n"
				+ "Co(ppm)");

		// Set bounds and add text area to the panel
		resultsArea.setBounds(0, 90, 220, 170);
		rightPanel.add(resultsArea);

		// Add buttons to the panel
		rightPanel.add(calculationButton());
		rightPanel.add(currentChartButton());
		rightPanel.add(permeationChartButton());
		
		// Create initial chart result text area
		chartResultsArea = new JTextArea(10, 10);
		chartResultsArea.append("CHART RESULTS" + "\n" + "Fick's Fit Deff"
				+ "\n" + "Ilno's Fit Deff" + "\n" + "Ilno's Kappa" + "\n"
				+ "Ilno's Lambda" + "\n" + "Ilno's My" + "\n"
				+ "Ilno's Residual" + "\n" + "Ilno's Runtime");

		// Set bounds and add text area to the panel
		chartResultsArea.setBounds(0, 360, 220, 130);
		rightPanel.add(chartResultsArea);

		// Add update button to the panel
		rightPanel.add(updateButton());

		// Set boundaries for the panel before returning it
		rightPanel.setBounds(Util.XSIZE / 2, 0, 800, 800);
		return rightPanel;

	}

	// Create text field for entering thickness
	public JTextField thicknessTextField() {
		
		// Initialize text field
		JTextField thicknessTextField = new JTextField("Enter specimen thickness", 20);

		// When initial background noise is entered
		thicknessTextField.addActionListener(new ActionListener() {

			public void actionPerformed(ActionEvent e) {
				try {
					L = Float.parseFloat(thicknessTextField.getText());
					resultsModel.setThickness(L);
				} catch (IndexOutOfBoundsException f) {
					thicknessTextField.setText("Wrong input format");
				}
			}
		});
		thicknessTextField.setBounds(0, 0, 150, 25);
		return thicknessTextField;
	}

	// Create text field for entering temperature
	public JTextField tempTextField() {
		
		// Initialize text field
		JTextField tempTextField = new JTextField("Enter T(C)", 20);

		// When temperature is entered
		tempTextField.addActionListener(new ActionListener() {

			public void actionPerformed(ActionEvent e) {
				try {
					temp = Float.parseFloat(tempTextField.getText());
					resultsModel.setDL(temp);
				} catch (IndexOutOfBoundsException f) {
					tempTextField.setText("Wrong input format");
				}
			}
		});
		tempTextField.setBounds(0, 30, 150, 25);
		return tempTextField;
	}

	// Create text field for entering initial noise
	public JTextField initialNoiseTextField() {
		
		// Initialize text field
		JTextField backgroundCurrentTextField = new JTextField("Enter background current",
				20);

		// When initial background noise is entered
		backgroundCurrentTextField.addActionListener(new ActionListener() {

			public void actionPerformed(ActionEvent e) {
				try {
					initialNoise = Float.parseFloat(backgroundCurrentTextField
							.getText());
					resultsModel.refreshTableData(initialNoise);
				} catch (IndexOutOfBoundsException f) {
					backgroundCurrentTextField.setText("Wrong input format");
				}
			}
		});
		backgroundCurrentTextField.setBounds(0, 60, 150, 25);
		return backgroundCurrentTextField;
	}

	// Create button for result calculation
	public JButton calculationButton() {
		JButton calculateButton = new JButton("Calculate");

		// When calculate button is pressed
		calculateButton.addActionListener(new ActionListener() {

			public void actionPerformed(ActionEvent e) {
				resultsModel.refreshResultsArea(resultsArea);
			}
		});
		calculateButton.setBounds(0, 270, 220, 25);
		return calculateButton;
	}

	// Create button for result calculation
	public JButton updateButton() {
		JButton updateButton = new JButton("Update");

		// When calculate button is pressed
		updateButton.addActionListener(new ActionListener() {

			public void actionPerformed(ActionEvent e) {
				if (currentRun && currentModel.getRun()) {
					
					// Rebuilding the original current list, from the currentModel current - Background list
					resultsModel.getCurrent().clear();
					for (int i = 0; i < currentModel.getCurrent().size(); i++) {
						resultsModel.getCurrent().add(currentModel.getCurrent().get(i) + initialNoise);
					}
					
					// Setting new time list and refreshing all values based on new lists
					resultsModel.setTime(currentModel.getTime());
					resultsModel.refreshTableData(initialNoise);
					resultsModel.refreshResultsArea(resultsArea);
					
					// Setting flag to update completed
					currentModel.setRun(false);
						
					// Refreshing Ilno's values
					if (currentModel.getIlnoRun()){
						System.out.println("CurrentIlno");
						currentModel.refreshResultsArea(chartResultsArea);
						currentModel.setIlnoRun(false);
					}
				} 
				if (fluxRun && permeationModel.getRun()){  
	
					// Rebuilding the original current and time list, from the permModel flux and tau lists
					resultsModel.getCurrent().clear();
					resultsModel.getTime().clear();
					for (int i = 0; i < permeationModel.getFlux().size(); i++) {
						resultsModel.getCurrent().add(permeationModel.getFlux().get(i)*resultsModel.getJss()*Util.FARADAY*Util.AREAL + initialNoise);
						resultsModel.getTime().add((permeationModel.getTau().get(i)*L*L)/resultsModel.getDl());
					}
					
					// Refreshing all values based on new lists
					resultsModel.refreshTableData(initialNoise);
					resultsModel.refreshResultsArea(resultsArea);
					
					// Setting flag to update completed
					permeationModel.setRun(false);
						
					// Refreshing Ilno's values
					if (permeationModel.getIlnoRun()) {
					    permeationModel.refreshResultsArea(chartResultsArea);
					    permeationModel.setIlnoRun(false);
						}
				}
			}
		});
		updateButton.setBounds(0, 500, 220, 25);
		return updateButton;
	}

	// Create button for JFREECHART current/time chart
	public JButton currentChartButton() {
		JButton currentChartGraphButton = new JButton("Permeation Current Chart");

		// When graph button is pressed
		currentChartGraphButton.addActionListener(new ActionListener() {

			public void actionPerformed(ActionEvent e) {
				currentRun = true;
				currentModel = null;
				currentModel = new CurrentChartModel(resultsModel.getTime(),
						resultsModel.getCurrentB(), L, resultsModel.getDeff(),
						resultsModel.getDl());
				new CurrentChartView("Permeation Current Plot - "
						+ resultsModel.getName(), currentModel);
			}
		});
		currentChartGraphButton.setBounds(0, 300, 220, 25);
		return currentChartGraphButton;
	}

	// Create button for JFREECHART permeation current chart
	public JButton permeationChartButton() {
		JButton permeationChartGraphButton = new JButton("Normalized Flux Chart");

		// When graph button is pressed
		permeationChartGraphButton.addActionListener(new ActionListener() {

			public void actionPerformed(ActionEvent e) {
				fluxRun = true;
				permeationModel = new PermeationChartModel(resultsModel.getTau(),
						resultsModel.getFlux(), L, resultsModel.getDeff(),
						resultsModel.getDl());
				new PermeationChartView("Normalized Flux Plot - "
						+ resultsModel.getName(), permeationModel);

			}

		});
		permeationChartGraphButton.setBounds(0, 330, 220, 25);
		return permeationChartGraphButton;
	}

}
