package diffusioncalculator;

import java.io.BufferedReader;
import java.io.FileWriter;
import java.io.IOException;
import java.util.ArrayList;

import org.jfree.data.xy.XYSeries;
import org.jfree.data.xy.XYSeriesCollection;

/* Uses the JFREECHART Library to make the data set plot in the Permeation chart. 
 * Also implements the low pass filter, and the high pass filter which removes data 
 * over/under a certain threshold. Lastly, it implements the restore button
 * which removes data in the chart, and loads the original content*/
public class PermeationChartModel {

	// Declaring lists containing data to be plotted
	public ArrayList<Float> tau;
	public ArrayList<Float> flux;

	// Declaring JFREECHART elements
	XYSeriesCollection dataset;
	XYSeries transient1;
	XYSeries transient2;
	XYSeries transient3;
	XYSeries ficksCurve;
	XYSeries ficksFit;
	XYSeries linosFit;

	// Read and write variables
	String line;
	String[] values;

	// Declaring locally used temporary variables and thickness
	float temp;
	double fick;
	float L;
	float deff;
	float R;
	float lastR;
	float a;
	double ls;
	int time;
	boolean flag;

	// Constructor initializing the data set used by the chart, taking data
	// arrays as inputs
	public PermeationChartModel(ArrayList<Float> inputTau,
			ArrayList<Float> inputFlux, float inputL, float inputDeff) {
		deff = inputDeff;
		L = inputL;
		tau = inputTau;
		flux = inputFlux;
		transient1 = new XYSeries("Transient 1");
		transient2 = new XYSeries("Transient 2");
		transient3 = new XYSeries("Transient 3");
		ficksCurve = new XYSeries("Fick's Curve");
		ficksFit = new XYSeries("Fick's Curve Fitting");
		linosFit = new XYSeries("Lino's Curve Fitting");

		// Adding x and y values to the series using the restore
		// function
		restore();
		//linostest();

		// Adding the series to the data set
		dataset = new XYSeriesCollection();
		dataset.addSeries(transient1);
		dataset.addSeries(ficksCurve);
	}

	public void linosFit() {
		linosFit.clear();
		dataset.removeSeries(linosFit);
		dataset.addSeries(linosFit);
		double linos = 0;
		double linos2 = 0;
		double linos3 = 0;
		double kappa = 1;
		double lambda = 10;
		double my = 0.1;
		double linos1 = (Math.sqrt(kappa))/(Math.sinh(Math.sqrt(kappa)));
		double[] snpos = new double[6];
		double[] snneg = new double[6];
		double sn1 = 0;
		double sn2 = 0;
		for(int n = 1; n < 7; n++){
			sn1 = n*n*Math.PI*Math.PI + kappa + lambda + my;
			sn2 = Math.sqrt(Math.pow(n*n*Math.PI*Math.PI + kappa + lambda - my,2) + 4*lambda*my);
			snpos[n-1] = 0.5*(sn1+sn2);
			snneg[n-1] = 0.5*(sn1-sn2);
		}
		temp = 0;
		for (double i = 1; i< tau.size(); i++){
			linos2 = -((2*Math.pow(1, 2)*Math.PI*Math.PI*Math.exp(-snneg[0]* ((3 * i * deff) /
					(L * L))))/(snneg[0]*(1+((lambda*my)/Math.pow(snneg[0]-my,2)))))
					+ ((2*Math.pow(2, 2)*Math.PI*Math.PI*Math.exp(-snneg[1]* ((3 * i * deff) /
					(L * L))))/(snneg[1]*(1+((lambda*my)/Math.pow(snneg[1]-my,2)))))
					- ((2*Math.pow(3, 2)*Math.PI*Math.PI*Math.exp(-snneg[2]* ((3 * i * deff) /
					(L * L))))/(snneg[2]*(1+((lambda*my)/Math.pow(snneg[2]-my,2)))))
					+ ((2*Math.pow(4, 2)*Math.PI*Math.PI*Math.exp(-snneg[3]* ((3 * i * deff) /
					(L * L))))/(snneg[3]*(1+((lambda*my)/Math.pow(snneg[3]-my,2)))))
					- ((2*Math.pow(5, 2)*Math.PI*Math.PI*Math.exp(-snneg[4]* ((3 * i * deff) /
					(L * L))))/(snneg[4]*(1+((lambda*my)/Math.pow(snneg[4]-my,2)))))
					+ ((2*Math.pow(6, 2)*Math.PI*Math.PI*Math.exp(-snneg[5]* ((3 * i * deff) /
					(L * L))))/(snneg[5]*(1+((lambda*my)/Math.pow(snneg[5]-my,2)))));
			
			linos3 = -((2*Math.pow(1, 2)*Math.PI*Math.PI*Math.exp(-snpos[0]* ((3 * i * deff) /
					(L * L))))/(snpos[0]*(1+((lambda*my)/Math.pow(snpos[0]-my,2)))))
					+ ((2*Math.pow(2, 2)*Math.PI*Math.PI*Math.exp(-snpos[1]* ((3 * i * deff) /
					(L * L))))/(snpos[1]*(1+((lambda*my)/Math.pow(snpos[1]-my,2)))))
					- ((2*Math.pow(3, 2)*Math.PI*Math.PI*Math.exp(-snpos[2]* ((3 * i * deff) /
					(L * L))))/(snpos[2]*(1+((lambda*my)/Math.pow(snpos[2]-my,2)))))
					+ ((2*Math.pow(4, 2)*Math.PI*Math.PI*Math.exp(-snpos[3]* ((3 * i * deff) /
					(L * L))))/(snpos[3]*(1+((lambda*my)/Math.pow(snpos[3]-my,2)))))
					- ((2*Math.pow(5, 2)*Math.PI*Math.PI*Math.exp(-snpos[4]* ((3 * i * deff) /
					(L * L))))/(snpos[4]*(1+((lambda*my)/Math.pow(snpos[4]-my,2)))))
					+ ((2*Math.pow(6, 2)*Math.PI*Math.PI*Math.exp(-snpos[5]* ((3 * i * deff) /
					(L * L))))/(snpos[5]*(1+((lambda*my)/Math.pow(snpos[5]-my,2)))));
			linos = linos1 + linos2 + linos3;
			temp = (tau.get((int) i));
			linosFit.add(temp, linos);
		}
	}
	
	public void linostest() {
		linosFit.clear();
		lastR = 200;
		flag = false;
		double linos = 0;
		double linos2 = 0;
		double linos3 = 0;
		double kappa = 0.001;
		double lambda = -0.3300000000000009;
		double my = 100;
		double linos1 = (Math.sqrt(kappa))/(Math.sinh(Math.sqrt(kappa)));
		double[] snpos = new double[6];
		double[] snneg = new double[6];
		double sn1 = 0;
		double sn2 = 0;
		temp = 0;
		while(!flag){
			my += 0.01;
		for(int n = 1; n < 7; n++){
			sn1 = n*n*Math.PI*Math.PI + kappa + lambda + my;
			sn2 = Math.sqrt(Math.pow(n*n*Math.PI*Math.PI + kappa + lambda - my,2) + 4*lambda*my);
			snpos[n-1] = 0.5*(sn1+sn2);
			snneg[n-1] = 0.5*(sn1-sn2);
		}
		R = 0;
		for (double i = 1; i< tau.size(); i++){
			linos2 = -((2*Math.pow(1, 2)*Math.PI*Math.PI*Math.exp(-snneg[0]* ((3 * i * deff) /
					(L * L))))/(snneg[0]*(1+((lambda*my)/Math.pow(snneg[0]-my,2)))))
					+ ((2*Math.pow(2, 2)*Math.PI*Math.PI*Math.exp(-snneg[1]* ((3 * i * deff) /
					(L * L))))/(snneg[1]*(1+((lambda*my)/Math.pow(snneg[1]-my,2)))))
					- ((2*Math.pow(3, 2)*Math.PI*Math.PI*Math.exp(-snneg[2]* ((3 * i * deff) /
					(L * L))))/(snneg[2]*(1+((lambda*my)/Math.pow(snneg[2]-my,2)))))
					+ ((2*Math.pow(4, 2)*Math.PI*Math.PI*Math.exp(-snneg[3]* ((3 * i * deff) /
					(L * L))))/(snneg[3]*(1+((lambda*my)/Math.pow(snneg[3]-my,2)))))
					- ((2*Math.pow(5, 2)*Math.PI*Math.PI*Math.exp(-snneg[4]* ((3 * i * deff) /
					(L * L))))/(snneg[4]*(1+((lambda*my)/Math.pow(snneg[4]-my,2)))))
					+ ((2*Math.pow(6, 2)*Math.PI*Math.PI*Math.exp(-snneg[5]* ((3 * i * deff) /
					(L * L))))/(snneg[5]*(1+((lambda*my)/Math.pow(snneg[5]-my,2)))));
			
			linos3 = -((2*Math.pow(1, 2)*Math.PI*Math.PI*Math.exp(-snpos[0]* ((3 * i * deff) /
					(L * L))))/(snpos[0]*(1+((lambda*my)/Math.pow(snpos[0]-my,2)))))
					+ ((2*Math.pow(2, 2)*Math.PI*Math.PI*Math.exp(-snpos[1]* ((3 * i * deff) /
					(L * L))))/(snpos[1]*(1+((lambda*my)/Math.pow(snpos[1]-my,2)))))
					- ((2*Math.pow(3, 2)*Math.PI*Math.PI*Math.exp(-snpos[2]* ((3 * i * deff) /
					(L * L))))/(snpos[2]*(1+((lambda*my)/Math.pow(snpos[2]-my,2)))))
					+ ((2*Math.pow(4, 2)*Math.PI*Math.PI*Math.exp(-snpos[3]* ((3 * i * deff) /
					(L * L))))/(snpos[3]*(1+((lambda*my)/Math.pow(snpos[3]-my,2)))))
					- ((2*Math.pow(5, 2)*Math.PI*Math.PI*Math.exp(-snpos[4]* ((3 * i * deff) /
					(L * L))))/(snpos[4]*(1+((lambda*my)/Math.pow(snpos[4]-my,2)))))
					+ ((2*Math.pow(6, 2)*Math.PI*Math.PI*Math.exp(-snpos[5]* ((3 * i * deff) /
					(L * L))))/(snpos[5]*(1+((lambda*my)/Math.pow(snpos[5]-my,2)))));
			linos = linos1 + linos2 + linos3;
			R += Math.pow((flux.get((int) i) - linos), 2);
		}
		System.out.println("R: " + R + " my: " + my);
		
		}
	}
	
	public void ficksFit() {
		ficksFit.clear();
		dataset.removeSeries(ficksFit);
		temp = 0;
		lastR = 200;
		a = (float) 15E-7;
		time = 0;
		flag = false;
		while (!flag){
			time++;
			a -= 9E-9;
			R = 0;
			for (double i = 1; i < tau.size(); i++) {
				R += Math.pow((flux.get((int) i) - (1 + 2 * (Math.exp(-36 * Math.PI * Math.PI * ((3 * i * a) / (L * L)))
					- Math.exp(-25 * Math.PI * Math.PI * ((3 * i * a) / (L * L)))
					+ Math.exp(-16 * Math.PI * Math.PI * ((3 * i * a) / (L * L)))
					- Math.exp(-9 * Math.PI * Math.PI * ((3 * i * a) / (L * L)))
					+ Math.exp(-4 * Math.PI * Math.PI * ((3 * i * a) / (L * L)))
					- Math.exp(-Math.PI * Math.PI * ((3 * i * a) / (L * L)))))),2);
			}
		if (lastR - R < 0){
			flag = true;
		}
		lastR = R;
		}
		for (double i = 1; i < tau.size(); i++) {
				ls = 1 + 2 * (Math.exp(-36 * Math.PI * Math.PI
				* ((3 * i * a) / (L * L)))
				- Math.exp(-25 * Math.PI * Math.PI
						* ((3 * i * a) / (L * L)))
				+ Math.exp(-16 * Math.PI * Math.PI
						* ((3 * i * a) / (L * L)))
				- Math.exp(-9 * Math.PI * Math.PI
						* ((3 * i * a) / (L * L)))
				+ Math.exp(-4 * Math.PI * Math.PI
						* ((3 * i * a) / (L * L))) - Math.exp(-Math.PI
				* Math.PI * ((3 * i * a) / (L * L))));
				temp = (tau.get((int) i));
				ficksFit.add(temp, ls);
		}
		dataset.addSeries(ficksFit);
	}

	// Implementing the low pass filter method
	public void lowPassFilter() {

		// Declaring a counter and holder variables for the last y value, and
		// the one
		// before there again
		float lastValue = 0f;
		float lastlastValue = 0f;
		int counter = 0;

		// Looking for abnormal values and replacing them with the last y value
		for (int i = 0; i < tau.size(); i++) {
			temp = (float) (transient1.getY(i));
			if (temp - lastValue > 0.02962) {
				transient1.remove(i);
				transient1.add(transient1.getX(i - 1), (double) (lastValue));
			} else if (temp - lastlastValue > 0.04) {
				transient1.remove(i);
				transient1.add(transient1.getX(i - 1), (double) (lastValue));
			} else {
				lastValue = temp;
				counter++;
			}
			if (counter == 2) {
				lastlastValue = lastValue;
				counter = 0;
			}

		}
	}

	// Implementing the high pass filter method
	public void highPassFilter() {

		// Declaring a counter and holder variables for the last y value, and
		// the one
		// before there again
		float lastValue = 0f;
		float lastlastValue = 0f;
		int counter = 0;

		// Looking for abnormal values and replacing them
		for (int i = 0; i < tau.size(); i++) {
			temp = (float) (transient1.getY(i));
			if (temp - lastValue < 0.02962) {
				transient1.remove(i);
				transient1.add(transient1.getX(i - 1), (double) (lastValue));
			} else if (temp - lastlastValue < 0.04) {
				transient1.remove(i);
				transient1.add(transient1.getX(i - 1), (double) (lastValue));
			} else {
				lastValue = temp;
				counter++;
			}
			if (counter == 2) {
				lastlastValue = lastValue;
				counter = 0;
			}

		}
	}

	// Restoring the original plot
	public void restore() {

		// Removing existing data and inputting new
		transient1.clear();
		transient2.clear();
		transient3.clear();
		ficksCurve.clear();
		ficksFit.clear();

		// Adding the main transient data
		for (int i = 0; i < tau.size(); i++) {
			transient1.add(tau.get(i), flux.get(i));
		}

		// Adding fick's curve data
		temp = 0;
		for (double i = 1; i < tau.size(); i++) {
			fick = 1 + 2 * (Math.exp(-36 * Math.PI * Math.PI
					* ((3 * i * deff) / (L * L)))
					- Math.exp(-25 * Math.PI * Math.PI
							* ((3 * i * deff) / (L * L)))
					+ Math.exp(-16 * Math.PI * Math.PI
							* ((3 * i * deff) / (L * L)))
					- Math.exp(-9 * Math.PI * Math.PI
							* ((3 * i * deff) / (L * L)))
					+ Math.exp(-4 * Math.PI * Math.PI
							* ((3 * i * deff) / (L * L))) - Math.exp(-Math.PI
					* Math.PI * ((3 * i * deff) / (L * L))));
			temp = (tau.get((int) i));
			ficksCurve.add(temp, fick);
		}
	}

	// Implementing writer method for transient 2
	public void transient2(BufferedReader br) throws IOException {
		transient2.clear();
		dataset.removeSeries(transient2);
		while ((line = br.readLine()) != null) {
			values = line.split("\t");
			transient2.add(Double.parseDouble(values[0]),
					Double.parseDouble(values[1]));
		}
		dataset.addSeries(transient2);
		br.close();
	}

	// Implementing writer method for transient 3
	public void transient3(BufferedReader br) throws IOException {
		transient3.clear();
		dataset.removeSeries(transient3);
		while ((line = br.readLine()) != null) {
			values = line.split("\t");
			transient3.add(Double.parseDouble(values[0]),
					Double.parseDouble(values[1]));
		}
		dataset.addSeries(transient3);
		br.close();
	}

	// Implementing a method to save series data locally, to be used later
	public void save(FileWriter fw) throws IOException {

		for (int i = 0; i < tau.size(); i++) {
			fw.write(transient1.getX(i) + "\t" + transient1.getY(i)
					+ System.getProperty("line.separator"));
		}
	}
}
