package diffusioncalculator;

import java.util.ArrayList;

import org.jfree.data.xy.XYSeries;
import org.jfree.data.xy.XYSeriesCollection;

/* Uses the JFREECHART Library to make the data set plot in the Current/Time chart. 
 * Also implements the low pass filter, and the high pass filter which removes data 
 * over/under a certain threshold. Lastly, it implements the restore button
 * which removes data in the chart, and loads the original content*/
public class CurrentChartModel {

	// Declaring lists containing data to be plotted
	public ArrayList<Float> time;
	public ArrayList<Float> current;

	// Declaring JFREECHART elements
	XYSeriesCollection dataset;
	XYSeries series;

	// Declaring locally used scale and temporary variables
	double scaleCurrent;
	float temp;

	// Constructor initializing the data set used by the chart, taking data
	// arrays as inputs
	public CurrentChartModel(ArrayList<Float> inputTime,
			ArrayList<Float> inputCurrent) {
		time = inputTime;
		current = inputCurrent;
		series = new XYSeries("Current/Time Curve");

		// Adding x and y values to the series using the restore
		// function implemented at the end of the class
		restore();
		//
		// Adding the series to the data set
		dataset = new XYSeriesCollection();
		dataset.addSeries(series);
	}

	// Implementing the low pass filter method
	public void lowPassFilter() {

		// Declaring holder variables for the last y value, and the one before
		// there again
		float lastValue = 0f;
		float lastlastValue = 0f;
		
		// Declaring a counter and a holder to help parse the y value
		int counter = 0;
		double db;
		
		
		// Looking for abnormal values and replacing them with the last y value
		for (int i = 0; i < time.size(); i++) {
			db = (double) (series.getY(i));
			temp = (float) db;
			if (temp - lastValue > 0.02962) {
				series.remove(i);
				series.add(series.getX(i - 1), (double) (lastValue));
			} else if (temp - lastlastValue > 0.04) {
				series.remove(i);
				series.add(series.getX(i - 1), (double) (lastValue));
			} else {
				lastValue = temp;
				counter++;
			}
			if (counter == 2) {
				lastlastValue = lastValue;
				counter = 0;
			}

		}
	}

	// Implementing the high pass filter method
	public void highPassFilter() {

		// Declaring holder variables for the last y value, and the one before
		// there again
		float lastValue = 0f;
		float lastlastValue = 0f;

		// Declaring a counter and a holder to help parse the y value
		int counter = 0;
		double db;

		// Looking for abnormal values and replacing them with the last y value
		for (int i = 0; i < time.size(); i++) {
			db = (double) (series.getY(i));
			temp = (float) db;
			if (temp - lastValue < 0.02962) {
				series.remove(i);
				series.add(series.getX(i - 1), (double) (lastValue));
			} else if (temp - lastlastValue < 0.04) {
				series.remove(i);
				series.add(series.getX(i - 1), (double) (lastValue));
			} else {
				lastValue = temp;
				counter++;
			}
			if (counter == 2) {
				lastlastValue = lastValue;
				counter = 0;
			}

		}
	}

	// Restoring the original plot
	public void restore() {
		
		//Removing existing data and inputting new
		series.clear();
		for (int i = 0; i < time.size(); i++) {
			
			// Scaling values to micro
			scaleCurrent = current.get(i) * 1000000;
			series.add((double) (time.get(i)), scaleCurrent);
		}
	}
}
