package diffusioncalculator;

import java.util.ArrayList;

import org.jfree.data.xy.XYSeries;
import org.jfree.data.xy.XYSeriesCollection;

/* Uses the JFREECHART Library to make the data set plot in the Permeation chart. 
 * Also implements the low pass filter, and the high pass filter which removes data 
 * over/under a certain threshold. Lastly, it implements the restore button
 * which removes data in the chart, and loads the original content*/
public class PermeationChartModel {

	// Declaring lists containing data to be plotted
	public ArrayList<Float> tau;
	public ArrayList<Float> flux;

	// Declaring JFREECHART elements
	XYSeriesCollection dataset;
	XYSeries series;

	// Declaring locally used temporary variables
	float temp;

	// Constructor initializing the data set used by the chart, taking data
	// arrays as inputs
	public PermeationChartModel(ArrayList<Float> inputTau,
			ArrayList<Float> inputFlux) {
		tau = inputTau;
		flux = inputFlux;
		series = new XYSeries("Permation Current Curve");

		// Adding x and y values to the series using the restore
		// function implemented at the end of the class
		restore();

		// Adding the series to the data set
		dataset = new XYSeriesCollection();
		dataset.addSeries(series);
	}

	// Implementing the low pass filter method
	public void lowPassFilter() {

		// Declaring a counter and holder variables for the last y value, and the one 
		// before there again
		float lastValue = 0f;
		float lastlastValue = 0f;
		int counter = 0;
		
		// Looking for abnormal values and replacing them with the last y value
		for (int i = 0; i < tau.size(); i++) {
			temp = (float) (series.getY(i));
			if (temp - lastValue > 0.02962) {
				series.remove(i);
				series.add(series.getX(i - 1), (double) (lastValue));
			} else if (temp - lastlastValue > 0.04) {
				series.remove(i);
				series.add(series.getX(i - 1), (double) (lastValue));
			} else {
				lastValue = temp;
				counter++;
			}
			if (counter == 2) {
				lastlastValue = lastValue;
				counter = 0;
			}

		}
	}

	// Implementing the high pass filter method
	public void highPassFilter() {
		
		// Declaring a counter and holder variables for the last y value, and the one 
		// before there again
		float lastValue = 0f;
		float lastlastValue = 0f;
		int counter = 0;
		
		// Looking for abnormal values and replacing them
		for (int i = 0; i < tau.size(); i++) {
			temp = (float) (series.getY(i));
			if (temp - lastValue < 0.02962) {
				series.remove(i);
				series.add(series.getX(i - 1), (double) (lastValue));
			} else if (temp - lastlastValue < 0.04) {
				series.remove(i);
				series.add(series.getX(i - 1), (double) (lastValue));
			} else {
				lastValue = temp;
				counter++;
			}
			if (counter == 2) {
				lastlastValue = lastValue;
				counter = 0;
			}

		}
	}

	// Restoring the original plot
	public void restore() {
		
		//Removing existing data and inputting new
		series.clear();
		for (int i = 0; i < tau.size(); i++) {
			series.add(tau.get(i), flux.get(i));
		}
	}

}
