package diffusioncalculator;

import java.awt.Color;
import java.awt.Dimension;

import javax.swing.JFrame;

import org.jfree.chart.ChartFactory;
import org.jfree.chart.ChartPanel;
import org.jfree.chart.JFreeChart;
import org.jfree.chart.axis.LogarithmicAxis;
import org.jfree.chart.plot.PlotOrientation;
import org.jfree.chart.plot.XYPlot;
import org.jfree.chart.renderer.xy.XYLineAndShapeRenderer;
import org.jfree.data.xy.XYDataset;

@SuppressWarnings("serial")
public class PermeationChartView extends JFrame {

	// Declaring the chart and panel
	JFreeChart chart;
	ChartPanel chartPanel;

	// Constructor creating the frame
	public PermeationChartView(final String title, PermeationChartModel permeationModel) {

		super(title);
		chart = createChart(permeationModel.dataset);
		chartPanel = new ChartPanel(chart);
		chartPanel.setPreferredSize(new Dimension((int) (Util.XSIZE * 0.95),
				(int) (Util.YSIZE * 0.95)));
		add(chartPanel);

	}

	// Creating the chart
	private JFreeChart createChart(final XYDataset dataset) {

		// Initializing the chart
		final JFreeChart chart = ChartFactory.createXYLineChart(
				"Permeation Plot", // chart title
				"tDl/L^2", // x axis label
				"J(t)/Jss", // y axis label
				dataset, // data
				PlotOrientation.VERTICAL, true, // include legend
				true, // tooltips
				false // urls
				);

		// Customization of the chart
		chart.setBackgroundPaint(Color.white);
		

		// Make a reference to the plot for further customization
		final XYPlot plot = chart.getXYPlot();
		plot.setBackgroundPaint(Color.lightGray);
		
		plot.setDomainGridlinePaint(Color.white);
		plot.setRangeGridlinePaint(Color.white);
		
		LogarithmicAxis xAxis = new LogarithmicAxis("tDl/L^2");
		plot.setDomainAxis(xAxis);
		
		// Rendering the line representation
		final XYLineAndShapeRenderer renderer = new XYLineAndShapeRenderer();
		renderer.setSeriesLinesVisible(0, true);
		renderer.setSeriesShapesVisible(0, false);
		plot.setRenderer(renderer);

		return chart;

	}

}