package diffusioncalculator;

import java.util.ArrayList;
import java.util.Collections;

import javax.swing.JTextArea;
import javax.swing.table.DefaultTableModel;

public class ResultsModel {

	// Declaring variables
	public ArrayList<Float> time;
	public ArrayList<Float> current;
	ArrayList<Float> currentB;
	ArrayList<Float> flux;
	ArrayList<Float> tau;
	Float[] rowNr;
	Float[] timeColumn;
	Float[] currentColumn;
	Float[] currentBColumn;
	Float[] fluxColumn;
	Float[] tauColumn;
	float initialNoise;
	float iss;
	float jss;
	int index;
	float tdl;
	float tlag;
	float deff;
	float comol;
	float cog;
	float coppm;
	float temp;
	float dl;
	double lastdiff;
	DefaultTableModel model;

	// Making a custom constructor
	public ResultsModel(ArrayList<Float> inputTime,
			ArrayList<Float> inputCurrent) {
		time = inputTime;
		current = inputCurrent;
	}

	// Make initial table data
	public void initialTableData() {

		// Creating model data
		model = new DefaultTableModel();

		// Declaring array size
		rowNr = new Float[time.size()];
		timeColumn = new Float[time.size()];
		currentColumn = new Float[current.size()];
		currentBColumn = new Float[current.size()];
		fluxColumn = new Float[current.size()];
		tauColumn = new Float[current.size()];

		// Making row numbers
		for (int i = 0; i < rowNr.length; i++) {
			rowNr[i] = (float) i;
		}

		// Converting data
		timeColumn = time.toArray(timeColumn);
		currentColumn = current.toArray(currentColumn);

		// Adding data to model
		model.addColumn("Nr.", rowNr);
		model.addColumn("Time", timeColumn);
		model.addColumn("Current", currentColumn);
		model.addColumn("Current - Background", currentBColumn);
		model.addColumn("J(t)/Jss", fluxColumn);
		model.addColumn("tDl/L^2", tauColumn);
	}

	// Calculating Dl from user's temperature input
	public void setDL(float inputTemp) {
		temp = (float) (273 + inputTemp);
		dl = (float) (Util.DLCOFF * Math.exp(-Util.Q / (Util.R * temp)));
	}

	// Making the remaining lists and computing results
	public void buttonCalculate() {

		// Initializing the last lists
		currentB = new ArrayList<Float>();
		flux = new ArrayList<Float>();
		tau = new ArrayList<Float>();

		// Initializing a difference variable and the index variable
		lastdiff = 1;
		index = 0;

		// Adding elements to the current list without background noise
		for (int i = 0; i < current.size(); i++) {
			currentB.add(current.get(i) - initialNoise);
		}

		// Finding the biggest value in the list
		iss = Collections.max(currentB);

		// Finding variable for further calculations
		jss = (iss / Util.AREAL) / Util.FARADAY;

		// Adding elements to the flux and tau lists
		for (int j = 0; j < currentB.size(); j++) {
			flux.add(((currentB.get(j) / Util.AREAL) / Util.FARADAY) / jss);
			tau.add((time.get(j) * dl) / Util.L2);

			// Finding index off J(t)/Jss closest to 0.63
			if (lastdiff > Math.abs(flux.get(j) - 0.63)) {
				lastdiff = Math.abs(flux.get(j) - 0.63);
				index = j;
			}
		}
	}

	// Refresh the data in the table
	public void refreshTableData(Float inputInitialNoise) {
		initialNoise = inputInitialNoise;
		// Calculating new values
		buttonCalculate();

		// Update data
		currentBColumn = currentB.toArray(currentBColumn);
		fluxColumn = flux.toArray(fluxColumn);
		tauColumn = tau.toArray(tauColumn);

		// Deleting existing data
		model.setColumnCount(0);

		// Adding columns
		model.addColumn("Nr.", rowNr);
		model.addColumn("Time", timeColumn);
		model.addColumn("Current", currentColumn);
		model.addColumn("Current - Background", currentBColumn);
		model.addColumn("J(t)/Jss", fluxColumn);
		model.addColumn("tDl/L^2", tauColumn);

		// Inform model that data has been updated
		model.fireTableDataChanged();
	}

	// Refresh results data
	public void refreshResultsArea(JTextArea resultsArea) {

		// remove initial data
		resultsArea.setText(null);

		// Save tau data at index (J(t)/Jss closest to 0.63)
		tdl = tau.get(index);

		// Computing remaining results
		tlag = (tdl * Util.L2) / dl;
		deff = Util.L2 / (6 * tlag);
		comol = (float) ((iss / 4.9) * Util.L) / (Util.FARADAY * deff);
		cog = comol * Util.HMASS;
		coppm = (float) ((cog / 7.9) * 1000000);

		// Append data to the result area
		resultsArea.append("RESULTS" + "\n" + "index" + "\t" + index + "\n"
				+ "DL" + "\t" + dl + "\n" + "tDl/L^2" + "\t" + tdl + "\n"
				+ "tlag" + "\t" + tlag + "\n" + "Deff" + "\t" + deff + "\n"
				+ "Co(mol/cm^3)" + "\t" + comol + "\n" + "Co(g/cm^3)" + "\t"
				+ cog + "\n" + "Co(ppm)" + "\t" + coppm);
	}
}
