package diffusioncalculator;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.util.ArrayList;

import javax.swing.JFileChooser;

import org.jfree.data.xy.XYSeries;
import org.jfree.data.xy.XYSeriesCollection;

/* Uses the JFREECHART Library to make the data set plot in the Current/Time chart. 
 * Also implements the low pass filter, and the high pass filter which removes data 
 * over/under a certain threshold. Lastly, it implements the restore button
 * which removes data in the chart, and loads the original content*/
public class CurrentChartModel {

	// Declaring lists containing data to be plotted
	public ArrayList<Float> time;
	public ArrayList<Float> current;

	// Declaring JFREECHART elements
	XYSeriesCollection dataset;
	XYSeries series1;
	XYSeries series2;
	XYSeries series3;

	// Declaring JFileChooser, reader, writer, and related variables
	JFileChooser fc;
	FileReader fr;
	FileWriter fw;
	File file;
	String line;
	String[] values;

	// Declaring locally used scale and temporary variables
	double scaleCurrent;
	float temp;

	// Constructor initializing the data set used by the chart, taking data
	// arrays as inputs
	public CurrentChartModel(ArrayList<Float> inputTime,
			ArrayList<Float> inputCurrent) throws IOException {
		time = inputTime;
		current = inputCurrent;
		series1 = new XYSeries("Transient 1");
		series2 = new XYSeries("Transient 2");

		// Adding x and y values to the series using the restore
		// function implemented at the end of the class
		restore();

		// Adding the series to the data set
		dataset = new XYSeriesCollection();
		dataset.addSeries(series1);
		save();
		write();
	}

	// Implementing the low pass filter method
	public void lowPassFilter() {

		// Declaring holder variables for the last y value, and the one before
		// there again
		float lastValue = 0f;
		float lastlastValue = 0f;

		// Declaring a counter and a holder to help parse the y value
		int counter = 0;
		double db;

		// Looking for abnormal values and replacing them with the last y value
		for (int i = 0; i < time.size(); i++) {
			db = (double) (series1.getY(i));
			temp = (float) db;
			if (temp - lastValue > 0.02962) {
				series1.remove(i);
				series1.add(series1.getX(i - 1), (double) (lastValue));
			} else if (temp - lastlastValue > 0.04) {
				series1.remove(i);
				series1.add(series1.getX(i - 1), (double) (lastValue));
			} else {
				lastValue = temp;
				counter++;
			}
			if (counter == 2) {
				lastlastValue = lastValue;
				counter = 0;
			}

		}
	}

	// Implementing the high pass filter method
	public void highPassFilter() {

		// Declaring holder variables for the last y value, and the one before
		// there again
		float lastValue = 0f;
		float lastlastValue = 0f;

		// Declaring a counter and a holder to help parse the y value
		int counter = 0;
		double db;

		// Looking for abnormal values and replacing them with the last y value
		for (int i = 0; i < time.size(); i++) {
			db = (double) (series1.getY(i));
			temp = (float) db;
			if (temp - lastValue < 0.02962) {
				series1.remove(i);
				series1.add(series1.getX(i - 1), (double) (lastValue));
			} else if (temp - lastlastValue < 0.04) {
				series1.remove(i);
				series1.add(series1.getX(i - 1), (double) (lastValue));
			} else {
				lastValue = temp;
				counter++;
			}
			if (counter == 2) {
				lastlastValue = lastValue;
				counter = 0;
			}

		}
	}

	// Restoring the original plot
	public void restore() {

		// Removing existing data and inputting new
		series1.clear();
		for (int i = 0; i < time.size(); i++) {

			// Scaling values to micro
			scaleCurrent = current.get(i) * 1000000;
			series1.add((double) (time.get(i)), scaleCurrent);
		}
	}
	
	// Implementing writer method, to apply a series from a file
		public void write() throws IOException {
			series2.clear();
			fc = new JFileChooser(System.getProperty("user.home") + File.separator
					+ "Desktop" + File.separator);
			if (fc.showOpenDialog(null) == JFileChooser.APPROVE_OPTION) {
				file = fc.getSelectedFile();
				fr = new FileReader(file);
				BufferedReader br = new BufferedReader(fr);
				while ((line = br.readLine()) != null) {
					values = line.split("\t");
					series2.add(Double.parseDouble(values[0]),
							Double.parseDouble(values[1]));
				}
				dataset.addSeries(series2);
				br.close();
			}
		}

		// Implementing a method to save series data locally, to be used later
		public void save() throws IOException {

			// Initializing JFileChooser
			fc = new JFileChooser(System.getProperty("user.home") + File.separator
					+ "Desktop" + File.separator);

			// Initializing the file writer, and writing the series to a .txt file
			fw = null;
			if (fc.showSaveDialog(null) == JFileChooser.APPROVE_OPTION) {
				try {
					fw = new FileWriter(fc.getSelectedFile() + ".txt");
					for (int i = 0; i < time.size(); i++) {
						fw.write(series1.getX(i) + "\t" + series1.getY(i)
								+ System.getProperty("line.separator"));
					}
				} catch (Exception ex) {
					ex.printStackTrace();
				}
			}

			// Closing the file writer
			fw.close();
		}
	
}
