package diffusioncalculator;

import java.io.BufferedReader;
import java.io.FileWriter;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Collections;

import javax.swing.JTextArea;

import org.jfree.data.xy.XYSeries;
import org.jfree.data.xy.XYSeriesCollection;

/* Uses the JFREECHART Library to make the data set plot in the Permeation chart. 
 * Also implements the low pass filter, and the high pass filter which removes data 
 * over/under a certain threshold. Lastly, it implements the restore button
 * which removes data in the chart, and loads the original content*/
public class PermeationChartModel {

	// Declaring lists containing data to be plotted
	public ArrayList<Float> tau;
	public ArrayList<Float> flux;

	// Declaring list used by Linos methods
	ArrayList<Double> endList;

	// Declaring JFREECHART elements
	XYSeriesCollection dataset;
	XYSeries transient1;
	XYSeries transient2;
	XYSeries transient3;
	XYSeries ficksCurve;
	XYSeries ficksElapsed;
	XYSeries ficksFit;
	XYSeries linosFit;

	// Declaring universal variables
	int index;
	float deff;
	float L;
	float dl;
	float ficksDeff;
	float linosDeff;
	double linosKappa;
	double linosLambda;
	double linosMy;
	double linosResidual;


	// Constructor initializing the data set used by the chart, taking data
	// arrays as inputs
	public PermeationChartModel(ArrayList<Float> inputTau,
			ArrayList<Float> inputFlux, float inputL, float inputDeff, float inputDl,
			int inputIndex) {
		index = inputIndex;
		deff = inputDeff;
		dl = inputDl;
		L = inputL;
		tau = inputTau;
		flux = inputFlux;
		transient1 = new XYSeries("Transient 1");
		transient2 = new XYSeries("Transient 2");
		transient3 = new XYSeries("Transient 3");
		ficksCurve = new XYSeries("Fick's Lattice Curve");
		ficksElapsed = new XYSeries("Fick's Elapsed Time Curve");
		ficksFit = new XYSeries("Fick's Curve Fitting");
		linosFit = new XYSeries("Lino's Curve Fitting");

		// Adding x and y values to the first transient using the restore
		// function
		restore();

		// Adding the series to the data set
		dataset = new XYSeriesCollection();
		dataset.addSeries(transient1);

	}

	public void ficksCurve() {
		ficksCurve.clear();
		dataset.removeSeries(ficksCurve);
		double temp;
		double ficks;
		for (double i = 1; i < tau.size(); i+=1) {
			ficks = 1 + 2 * (Math.exp(-36 * Math.PI * Math.PI
					* ((i * dl) / (L * L)))
					- Math.exp(-25 * Math.PI * Math.PI
							* ((i * dl) / (L * L)))
					+ Math.exp(-16 * Math.PI * Math.PI
							* ((i * dl) / (L * L)))
					- Math.exp(-9 * Math.PI * Math.PI
							* ((i * dl) / (L * L)))
					+ Math.exp(-4 * Math.PI * Math.PI
							* ((i * dl) / (L * L))) - Math
					.exp(-Math.PI * Math.PI * ((i * dl) / (L * L))));
			temp = ((i * dl) / (L * L));
			ficksCurve.add(temp, ficks);
		}
		dataset.addSeries(ficksCurve);
	}
	
	public void ficksElapsed() {
		ficksElapsed.clear();
		dataset.removeSeries(ficksElapsed);
		// Adding fick's curve data
		double temp;
		double ficks;
		for (double i = 1; i < tau.size(); i++) {
			ficks = 1 + 2 * (Math.exp(-36 * Math.PI * Math.PI
					* ((3 * i * deff) / (L * L)))
					- Math.exp(-25 * Math.PI * Math.PI
							* ((3 * i * deff) / (L * L)))
					+ Math.exp(-16 * Math.PI * Math.PI
							* ((3 * i * deff) / (L * L)))
					- Math.exp(-9 * Math.PI * Math.PI
							* ((3 * i * deff) / (L * L)))
					+ Math.exp(-4 * Math.PI * Math.PI
							* ((3 * i * deff) / (L * L))) - Math.exp(-Math.PI
					* Math.PI * ((3 * i * deff) / (L * L))));
			temp = (tau.get((int) i));
			ficksElapsed.add(temp, ficks);
		}
		dataset.addSeries(ficksElapsed);
	}
	
	public double linosMethod(double[] snneg, double[] snpos, double lambda,
			double my, double i, double deff) {
		double linos2 = -((2 * Math.pow(1, 2) * Math.PI * Math.PI * Math
				.exp(-snneg[0] * ((3 * i * deff) / (L * L)))) / (snneg[0] * (1 + ((lambda * my) / Math
				.pow(snneg[0] - my, 2)))))
				+ ((2 * Math.pow(2, 2) * Math.PI * Math.PI * Math.exp(-snneg[1]
						* ((3 * i * deff) / (L * L)))) / (snneg[1] * (1 + ((lambda * my) / Math
						.pow(snneg[1] - my, 2)))))
				- ((2 * Math.pow(3, 2) * Math.PI * Math.PI * Math.exp(-snneg[2]
						* ((3 * i * deff) / (L * L)))) / (snneg[2] * (1 + ((lambda * my) / Math
						.pow(snneg[2] - my, 2)))))
				+ ((2 * Math.pow(4, 2) * Math.PI * Math.PI * Math.exp(-snneg[3]
						* ((3 * i * deff) / (L * L)))) / (snneg[3] * (1 + ((lambda * my) / Math
						.pow(snneg[3] - my, 2)))))
				- ((2 * Math.pow(5, 2) * Math.PI * Math.PI * Math.exp(-snneg[4]
						* ((3 * i * deff) / (L * L)))) / (snneg[4] * (1 + ((lambda * my) / Math
						.pow(snneg[4] - my, 2)))))
				+ ((2 * Math.pow(6, 2) * Math.PI * Math.PI * Math.exp(-snneg[5]
						* ((3 * i * deff) / (L * L)))) / (snneg[5] * (1 + ((lambda * my) / Math
						.pow(snneg[5] - my, 2)))));

		double linos3 = -((2 * Math.pow(1, 2) * Math.PI * Math.PI * Math
				.exp(-snpos[0] * ((3 * i * deff) / (L * L)))) / (snpos[0] * (1 + ((lambda * my) / Math
				.pow(snpos[0] - my, 2)))))
				+ ((2 * Math.pow(2, 2) * Math.PI * Math.PI * Math.exp(-snpos[1]
						* ((3 * i * deff) / (L * L)))) / (snpos[1] * (1 + ((lambda * my) / Math
						.pow(snpos[1] - my, 2)))))
				- ((2 * Math.pow(3, 2) * Math.PI * Math.PI * Math.exp(-snpos[2]
						* ((3 * i * deff) / (L * L)))) / (snpos[2] * (1 + ((lambda * my) / Math
						.pow(snpos[2] - my, 2)))))
				+ ((2 * Math.pow(4, 2) * Math.PI * Math.PI * Math.exp(-snpos[3]
						* ((3 * i * deff) / (L * L)))) / (snpos[3] * (1 + ((lambda * my) / Math
						.pow(snpos[3] - my, 2)))))
				- ((2 * Math.pow(5, 2) * Math.PI * Math.PI * Math.exp(-snpos[4]
						* ((3 * i * deff) / (L * L)))) / (snpos[4] * (1 + ((lambda * my) / Math
						.pow(snpos[4] - my, 2)))))
				+ ((2 * Math.pow(6, 2) * Math.PI * Math.PI * Math.exp(-snpos[5]
						* ((3 * i * deff) / (L * L)))) / (snpos[5] * (1 + ((lambda * my) / Math
						.pow(snpos[5] - my, 2)))));
		return linos2 + linos3;
	}


	public void linosFit() {
		int imax = flux.indexOf(Collections.max(flux));
		endList = new ArrayList<Double>();
		linosFit.clear();
		dataset.removeSeries(linosFit);
		double linos;
		double kappa = 0.0001;
		double lambda = 5;
		double my = 1;
		double newDeff = 0;
		double[] snpos = new double[6];
		double[] snneg = new double[6];
		double sn1;
		double sn2;
		double temp;
		double newMy = 0;
		double newLambda = 0;
		double R = 10;
//		newDeff = 9.575253398467439E-5;
//		int counter = 0;
		for (lambda = 0; lambda < 500; lambda+=1){
			newDeff = linosDeff(kappa, lambda, my, imax);
//			System.out.println(" R: " + endList.get(counter) + " newMy: " + newMy + " newDeff: " + newDeff + " lambda: " + lambda);
//			counter++;
	}
		double last = 200;
		for (int a = 0; a<endList.size();a+=2){
			if (endList.get(a)<last){
				newDeff = endList.get(a+1);	
				last = endList.get(a);
				}
			}
		endList.clear();
//		System.exit(1);
		//newDeff = linosDeff(kappa,lambda,my, imax);
		System.out.println(newDeff);
		for (lambda = 1; lambda < 500; lambda += 1) {
//			newDeff = linosDeff(kappa, lambda, my, imax);
			System.out.println(lambda);
			newMy = linosMy(kappa, lambda, my, newDeff, imax);
			newLambda = lambda;
			endList.add(newDeff);
			endList.add(newMy);
			endList.add(newLambda);
		}
		for (int k = 0; k < endList.size(); k += 4) {
			 System.out.println("nr: " + k/3 + " R: " + endList.get(k) +
			 " deff: " + endList.get(k+1) + " my: " + endList.get(k+2) +
			 " lambda: " + endList.get(k+3) + " kappa: " + kappa );
			if (endList.get(k) < R) {
				R = endList.get(k);
				newDeff = endList.get(k + 1);
				newMy = endList.get(k + 2);
				newLambda = endList.get(k + 3);
			}
		}
		double linos1 = (Math.sqrt(kappa)) / (Math.sinh(Math.sqrt(kappa)));
		for (int n = 1; n < 7; n++) {
			sn1 = n * n * Math.PI * Math.PI + kappa + newLambda + newMy;
			sn2 = Math.sqrt(Math.pow(n * n * Math.PI * Math.PI + kappa
					+ newLambda - newMy, 2)
					+ 4 * newLambda * newMy);
			snpos[n - 1] = 0.5 * (sn1 + sn2);
			snneg[n - 1] = 0.5 * (sn1 - sn2);
		}
		 System.out.println("linos1: " + linos1 + " R: " + R + " newDeff: " +
		 newDeff + " newMy: " + newMy + " newLambda: " + newLambda +
		 " newKappa: " + kappa);
		 linosDeff = (float) newDeff;
		 linosMy = newMy;
		 linosLambda = newLambda;
		 linosKappa = kappa;
		 linosResidual = R;
		for (int i = (int) (0.1 * tau.size()); i < imax; i++) {
			linos = linos1
					+ linosMethod(snneg, snpos, newLambda, newMy, i, newDeff);
			temp = (tau.get((int) i));
			linosFit.add(temp, linos);
		}
		dataset.addSeries(linosFit);
	}

	public double linosDeff(double kappa, double lambda, double my, int imax) {
		ArrayList<Double> rList = new ArrayList<Double>();
		ArrayList<Double> list = new ArrayList<Double>();
		double sn1;
		double sn2;
		double[] snpos = new double[6];
		double[] snneg = new double[6];
		double linos1 = (Math.sqrt(kappa)) / (Math.sinh(Math.sqrt(kappa)));
		double linos;
		double R = 0;
		for (int n = 1; n < 7; n++) {
			sn1 = n * n * Math.PI * Math.PI + kappa + lambda + my;
			sn2 = Math.sqrt(Math.pow(n * n * Math.PI * Math.PI + kappa + lambda
					- my, 2)
					+ 4 * lambda * my);
			snpos[n - 1] = 0.5 * (sn1 + sn2);
			snneg[n - 1] = 0.5 * (sn1 - sn2);
		}

		for (double newDeff = deff/10; newDeff<10*deff;newDeff+=1E-9) {
			R = 0;
			for (int i = (int) (0.1 * tau.size()); i < imax; i++) {
				linos = linos1
						+ linosMethod(snneg, snpos, lambda, my, i, newDeff);
				R += Math.pow((flux.get((int) i) - linos), 2);
			}
			//System.out.println("R: " + R + " newDeff: " + newDeff + " my: " + my);
			rList.add((double) R);
			list.add(newDeff);
		}
		// System.out.println("R: " + R + " newDeff: " + newDeff);
		int minIndex = rList.indexOf(Collections.min(rList));
		// System.out.println("min rList: " + Collections.min(rList) +
		// " index: " + rList.indexOf(Collections.min(rList)) + " deff: " +
		// list.get(minIndex));
		endList.add(Collections.min(rList));
		endList.add(list.get(minIndex));
		return list.get(minIndex);
	}

	public double linosMy(double kappa, double lambda, double my, double newDeff, int imax) {
		ArrayList<Double> rList = new ArrayList<Double>();
		ArrayList<Double> list = new ArrayList<Double>();
		double sn1;
		double sn2;
		double[] snpos = new double[6];
		double[] snneg = new double[6];
		double linos1 = (Math.sqrt(kappa)) / (Math.sinh(Math.sqrt(kappa)));
		double linos;
		float R = 0;
		for (my = 0; my < 50; my += 0.1) {
			for (int n = 1; n < 7; n++) {
				sn1 = n * n * Math.PI * Math.PI + kappa + lambda + my;
				sn2 = Math.sqrt(Math.pow(n * n * Math.PI * Math.PI + kappa
						+ lambda - my, 2)
						+ 4 * lambda * my);
				snpos[n - 1] = 0.5 * (sn1 + sn2);
				snneg[n - 1] = 0.5 * (sn1 - sn2);
			}
			R = 0;
			for (int i = 0; i < imax; i++) {
				linos = linos1
						+ linosMethod(snneg, snpos, lambda, my, i, newDeff);
				R += Math.pow((flux.get((int) i) - linos), 2);
			}
			rList.add((double) R);
			list.add(my);
		}
		int minIndex = rList.indexOf(Collections.min(rList));
		endList.add(Collections.min(rList));
		return list.get(minIndex);
	}
	
	public void linosLow() {
		// declaring temporary holder variables
		double lino;
	    float trans;
		float average;
		boolean flag = true;
	
		// Looking for abnormal values and replacing them with the lino value
		while (flag) {
			flag = false;
			for (int i = (int) (tau.size() * 0.2); i < tau.size()-1; i++) {
				trans = (float) (transient1.getY(i));
				lino = (double) (linosFit.getY(i));
				if (trans - lino > 0.02962) {
					flag = true;
					transient1.remove(i);
					average = ((float) transient1.getY(i - 1) + (float) transient1
							.getY(i + 1)) / 2;
					transient1.add(transient1.getX(i), average);
					flux.set(i, (float) (average));
				}

			}
		}
	}

	public void linosHigh() {
		// declaring temporary holder variables
		double lino;
		float trans;
		float average;
		boolean flag = true;
		// Looking for abnormal values and replacing them with the lino value
		while (flag) {
			flag = false;
			for (int i = (int) (tau.size() * 0.2); i < tau.size()-1; i++) {
				trans = (float) (transient1.getY(i));
				lino = (double) (linosFit.getY(i));
				if (trans - lino < 0.02962) {
					flag = true;
					transient1.remove(i);
					average = ((float) transient1.getY(i - 1) + (float) transient1
							.getY(i + 1)) / 2;
					transient1.add(transient1.getX(i), average);
					flux.set(i, (float) (average));
				}

			}
		}
	}

	public void ficksFit() {
		int imax = flux.indexOf(Collections.max(flux));
		ficksFit.clear();
		dataset.removeSeries(ficksFit);
		double temp;
		double lastR = 200;
		double newDeff = deff;
		boolean flag = false;
		double R;
		double ficks;
		while (!flag) {
			newDeff -= 9E-9;
			R = 0;
			for (double i = (int) (0.1 * tau.size()); i < imax; i++) {
				R += Math.pow(
						(flux.get((int) i) - (1 + 2 * (Math.exp(-36 * Math.PI
								* Math.PI * ((3 * i * newDeff) / (L * L)))
								- Math.exp(-25 * Math.PI * Math.PI
										* ((3 * i * newDeff) / (L * L)))
								+ Math.exp(-16 * Math.PI * Math.PI
										* ((3 * i * newDeff) / (L * L)))
								- Math.exp(-9 * Math.PI * Math.PI
										* ((3 * i * newDeff) / (L * L)))
								+ Math.exp(-4 * Math.PI * Math.PI
										* ((3 * i * newDeff) / (L * L))) - Math
								.exp(-Math.PI * Math.PI
										* ((3 * i * newDeff) / (L * L)))))), 2);
			}
			if (lastR - R < 0) {
				flag = true;
			}
			lastR = R;
		}
		ficksDeff = (float) newDeff;
		for (double i = 1; i < tau.size(); i++) {
			ficks = 1 + 2 * (Math.exp(-36 * Math.PI * Math.PI
					* ((3 * i * newDeff) / (L * L)))
					- Math.exp(-25 * Math.PI * Math.PI
							* ((3 * i * newDeff) / (L * L)))
					+ Math.exp(-16 * Math.PI * Math.PI
							* ((3 * i * newDeff) / (L * L)))
					- Math.exp(-9 * Math.PI * Math.PI
							* ((3 * i * newDeff) / (L * L)))
					+ Math.exp(-4 * Math.PI * Math.PI
							* ((3 * i * newDeff) / (L * L))) - Math
					.exp(-Math.PI * Math.PI * ((3 * i * newDeff) / (L * L))));
			temp = (tau.get((int) i));
			ficksFit.add(temp, ficks);
		}
		dataset.addSeries(ficksFit);
	}

	// Implementing the low pass filter method
	public void lowPassFilter() {

		// Declaring a counter and holder variables for the last y value, and
		// the one
		// before there again
		float lastValue = 0f;
		float lastlastValue = 0f;
		int counter = 0;
		float temp;

		// Looking for abnormal values and replacing them with the last y value
		for (int i = 0; i < tau.size(); i++) {
			temp = (float) (transient1.getY(i));
			if (temp - lastValue > 0.02962) {
				transient1.remove(i);
				transient1.add(transient1.getX(i - 1), (double) (lastValue));
			} else if (temp - lastlastValue > 0.04) {
				transient1.remove(i);
				transient1.add(transient1.getX(i - 1), (double) (lastValue));
			} else {
				lastValue = temp;
				counter++;
			}
			if (counter == 2) {
				lastlastValue = lastValue;
				counter = 0;
			}

		}
	}

	// Implementing the high pass filter method
	public void highPassFilter() {

		// Declaring a counter and holder variables for the last y value, and
		// the one
		// before there again
		float lastValue = 0f;
		float lastlastValue = 0f;
		int counter = 0;
		float temp;

		// Looking for abnormal values and replacing them
		for (int i = 0; i < tau.size(); i++) {
			temp = (float) (transient1.getY(i));
			if (temp - lastValue < 0.02962) {
				transient1.remove(i);
				transient1.add(transient1.getX(i - 1), (double) (lastValue));
			} else if (temp - lastlastValue < 0.04) {
				transient1.remove(i);
				transient1.add(transient1.getX(i - 1), (double) (lastValue));
			} else {
				lastValue = temp;
				counter++;
			}
			if (counter == 2) {
				lastlastValue = lastValue;
				counter = 0;
			}

		}
	}

	// Restoring the original plot
	public void restore() {

		// Removing existing data and inputting new
		transient1.clear();
		transient2.clear();
		transient3.clear();
		ficksElapsed.clear();
		ficksFit.clear();
		ficksCurve.clear();
		linosFit.clear();

		// Adding the main transient data
		for (int i = 0; i < tau.size(); i++) {
			transient1.add(tau.get(i), flux.get(i));
		}
	}

	// Implementing writer method for transient 2
	public void transient2(BufferedReader br) throws IOException {
		transient2.clear();
		dataset.removeSeries(transient2);
		String line;
		String[] values;
		while ((line = br.readLine()) != null) {
			values = line.split("\t");
			transient2.add(Double.parseDouble(values[0]),
					Double.parseDouble(values[1]));
		}
		dataset.addSeries(transient2);
		br.close();
	}

	// Implementing writer method for transient 3
	public void transient3(BufferedReader br) throws IOException {
		transient3.clear();
		dataset.removeSeries(transient3);
		String line;
		String[] values;
		while ((line = br.readLine()) != null) {
			values = line.split("\t");
			transient3.add(Double.parseDouble(values[0]),
					Double.parseDouble(values[1]));
		}
		dataset.addSeries(transient3);
		br.close();
	}

	// Implementing a method to save series data locally, to be used later
	public void save(FileWriter fw) throws IOException {

		for (int i = 0; i < tau.size(); i++) {
			fw.write(transient1.getX(i) + "\t" + transient1.getY(i)
					+ System.getProperty("line.separator"));
		}
	}
	
	public void refreshResultsArea(JTextArea resultsArea)  {
		// remove initial data
				resultsArea.setText(null);

				// Append data to the result area
				resultsArea.append("CHART RESULTS" + "\n" + "Ficks Deff" + "\t" + ficksDeff + "\n"
						+ "Linos Deff" + "\t" + linosDeff + "\n" + "Linos Kappa" + "\t" + linosKappa +  "\n" + "Linos Lambda" + "\t" + linosLambda + "\n"
						+ "Linos My" + "\t" + linosMy + "\n" + "Linos Residual" + "\t" + linosResidual);
				


				
	}
	
}
