package diffusioncalculator;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.JTextArea;
import javax.swing.JTextField;

/* Class creating main view of the program, letting the
 * user input parameters, calculate values and plot charts.
 * Showing a scroll table with all data read from file,
 * and columns calculated from this data. */
public class ResultsView {

	// Declaring ResultsModel instance
	ResultsModel resultsModel;

	// Initializing charts and initiater
	PermeationChartModel permeationModel;
	CurrentChartModel currentModel;
	Boolean permRun = false;
	Boolean currentRun = false;

	// Declaring GUI components
	JTextArea textArea;
	JPanel rightPanel;
	JTextArea resultsArea;
	JTextArea chartResultsArea;
	JTable table;
	JTextField backgroundCurrentTextField;
	JTextField tempTextField;
	JTextField thicknessTextField;
	JButton calculateButton;
	JButton updateButton;
	JButton currentChartGraphButton;
	JButton permeationChartGraphButton;
	JScrollPane scrollableTable;

	// Declaring length of specimen, initialNoise, and temperature
	// input from user
	float L;
	float initialNoise;
	float temp;

	// Constructor declaring and adding components to a frame.
	public ResultsView(ResultsModel inputResultsModel) {

		// Initializing a results model
		resultsModel = inputResultsModel;

		// Create and set up the window.
		final JFrame frame = new JFrame(resultsModel.name);

		// Make the program end when the window is closed
		frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);

		// Set default layout
		frame.setLayout(null);

		// Add content
		frame.add(leftScrollTable());
		frame.add(rightPanel());

		// Set dimensions and make the frame visible
		frame.setSize((int) (Util.XSIZE * 0.7), (int) (Util.YSIZE * 0.95));
		frame.setVisible(true);

	}

	// Create scroll pane with table
	public JScrollPane leftScrollTable() {

		// Create initial data input
		resultsModel.initialTableData();

		// Initialize table with data
		table = new JTable(resultsModel.model);

		// Add scroll pane to table
		scrollableTable = new JScrollPane(table);
		scrollableTable
				.setHorizontalScrollBarPolicy(JScrollPane.HORIZONTAL_SCROLLBAR_ALWAYS);
		scrollableTable
				.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_ALWAYS);

		// Set boundaries and return table
		scrollableTable.setBounds(0, 0, Util.XSIZE / 2, Util.YSIZE - 70);
		return scrollableTable;
	}

	// Create right panel
	public JPanel rightPanel() {

		// create GUI components
		thicknessTextField();
		initialNoiseTextField();
		tempTextField();
		calculationButton();
		currentChartButton();
		permeationChartButton();
		updateButton();

		// Create initial result text area
		resultsArea = new JTextArea(10, 10);
		resultsArea.append("RESULTS" + "\n" + "index" + "\n" + "Dl" + "\n"
				+ "imax" + "\n" + "tDl/L^2" + "\n" + "tlag" + "\n" + "Deff"
				+ "\n" + "Co(mol/cm^3)" + "\n" + "Co(g/cm^3)" + "\n"
				+ "Co(ppm)");

		// Declare panel and set default layout
		rightPanel = new JPanel();
		rightPanel.setLayout(null);

		// Set boundaries and add components to the panel
		thicknessTextField.setBounds(0, 0, 150, 25);
		rightPanel.add(thicknessTextField);
		tempTextField.setBounds(0, 30, 150, 25);
		rightPanel.add(tempTextField);
		backgroundCurrentTextField.setBounds(0, 60, 150, 25);
		rightPanel.add(backgroundCurrentTextField);

		resultsArea.setBounds(0, 90, 220, 170);
		rightPanel.add(resultsArea);

		calculateButton.setBounds(0, 270, 220, 25);
		rightPanel.add(calculateButton);

		currentChartGraphButton.setBounds(0, 300, 220, 25);
		rightPanel.add(currentChartGraphButton);
		permeationChartGraphButton.setBounds(0, 330, 220, 25);
		rightPanel.add(permeationChartGraphButton);

		// Create initial chart result text area
		chartResultsArea = new JTextArea(10, 10);
		chartResultsArea.append("CHART RESULTS" + "\n" + "Ficks Deff" + "\n"
				+ "Linos Deff" + "\n" + "Linos Kappa" + "\n" + "Linos Lambda"
				+ "\n" + "Linos My" + "\n" + "Linos Residual" + "\n"
				+ "Time");

		chartResultsArea.setBounds(0, 360, 220, 130);
		rightPanel.add(chartResultsArea);

		updateButton.setBounds(0, 500, 220, 25);
		rightPanel.add(updateButton);

		// Set boundaries for the panel before returning it
		rightPanel.setBounds(Util.XSIZE / 2, 0, 800, 800);
		return rightPanel;

	}

	// Create text field for entering thickness
	public void thicknessTextField() {
		thicknessTextField = new JTextField("Enter specimen thickness", 20);

		// When initial background noise is entered
		thicknessTextField.addActionListener(new ActionListener() {

			public void actionPerformed(ActionEvent e) {
				try {
					L = Float.parseFloat(thicknessTextField.getText());
					resultsModel.setThickness(L);
				} catch (IndexOutOfBoundsException f) {
					thicknessTextField.setText("Wrong input format");
				}
			}
		});
	}

	// Create text field for entering temperature
	public void tempTextField() {
		tempTextField = new JTextField("Enter T(C)", 20);

		// When temperature is entered
		tempTextField.addActionListener(new ActionListener() {

			public void actionPerformed(ActionEvent e) {
				try {
					temp = Float.parseFloat(tempTextField.getText());
					resultsModel.setDL(temp);
				} catch (IndexOutOfBoundsException f) {
					tempTextField.setText("Wrong input format");
				}
			}
		});
	}

	// Create text field for entering initial noise
	public void initialNoiseTextField() {
		backgroundCurrentTextField = new JTextField("Enter background current",
				20);

		// When initial background noise is entered
		backgroundCurrentTextField.addActionListener(new ActionListener() {

			public void actionPerformed(ActionEvent e) {
				try {
					initialNoise = Float.parseFloat(backgroundCurrentTextField
							.getText());
					resultsModel.refreshTableData(initialNoise);
				} catch (IndexOutOfBoundsException f) {
					backgroundCurrentTextField.setText("Wrong input format");
				}
			}
		});
	}

	// Create button for result calculation
	public void calculationButton() {
		calculateButton = new JButton("Calculate");

		// When calculate button is pressed
		calculateButton.addActionListener(new ActionListener() {

			public void actionPerformed(ActionEvent e) {
				resultsModel.refreshResultsArea(resultsArea);
			}
		});
	}

	// Create button for result calculation
	public void updateButton() {
		updateButton = new JButton("Update");

		// When calculate button is pressed
		updateButton.addActionListener(new ActionListener() {

			public void actionPerformed(ActionEvent e) {
				if (currentRun && currentModel.linoRun) {
					currentModel.refreshResultsArea(chartResultsArea);
				} else if (permRun && permeationModel.linoRun) {
					permeationModel.refreshResultsArea(chartResultsArea);
				}
			}
		});
	}

	// Create button for JFREECHART current/time chart
	public void currentChartButton() {
		currentChartGraphButton = new JButton("Make current/time chart");

		// When graph button is pressed
		currentChartGraphButton.addActionListener(new ActionListener() {

			public void actionPerformed(ActionEvent e) {
				currentModel = null;
				currentRun = true;
				currentModel = new CurrentChartModel(resultsModel.time,
						resultsModel.currentB, L, resultsModel.getDeff(),
						resultsModel.getDl(), resultsModel.getIndex());
				new CurrentChartView("Current/Time Plot - " + resultsModel.name, currentModel);
			}
		});
	}

	// Create button for JFREECHART permeation current chart
	public void permeationChartButton() {
		permeationChartGraphButton = new JButton("Make permeation chart");

		// When graph button is pressed
		permeationChartGraphButton.addActionListener(new ActionListener() {

			public void actionPerformed(ActionEvent e) {
				permRun = true;
				permeationModel = new PermeationChartModel(resultsModel.tau,
						resultsModel.flux, L, resultsModel.getDeff(),
						resultsModel.getDl(), resultsModel.getIndex());
				new PermeationChartView("Permeation Current Plot - " + resultsModel.name,
						permeationModel);

			}

		});
	}

}
