package diffusioncalculator;

import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseEvent;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.awt.geom.Rectangle2D;
import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;

import javax.swing.JButton;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JPanel;

import org.jfree.chart.ChartFactory;
import org.jfree.chart.ChartMouseEvent;
import org.jfree.chart.ChartMouseListener;
import org.jfree.chart.ChartPanel;
import org.jfree.chart.JFreeChart;
import org.jfree.chart.axis.ValueAxis;
import org.jfree.chart.entity.XYItemEntity;
import org.jfree.chart.panel.CrosshairOverlay;
import org.jfree.chart.plot.Crosshair;
import org.jfree.chart.plot.PlotOrientation;
import org.jfree.chart.plot.XYPlot;
import org.jfree.chart.renderer.xy.XYLineAndShapeRenderer;
import org.jfree.data.general.DatasetUtilities;
import org.jfree.data.xy.XYDataset;
import org.jfree.ui.RectangleEdge;

/* Class making all GUI concerning the Current/Time chart. 
 * Setting up layouts, applying crosshairs and making components.
 * Functions used to create the chart, buttons, and registering 
 * mouse activity*/
@SuppressWarnings("serial")
public class CurrentChartView extends JFrame implements ChartMouseListener {

	// Declaring the JFREECHART chart and panel
	JFreeChart chart;
	ChartPanel chartPanel;

	// Declaring buttons and panel to attach button onto
	JButton lowPassButton;
	JButton highPassButton;
	JButton restoreButton;
	JButton transient2Button;
	JButton transient3Button;
	JButton saveButton;
	JButton ficksCurveButton;
	JButton ficksElapsedButton;
	JButton ficksFitButton;
	JButton ilnosFitButton;
	JButton ilnosLowButton;
	JButton ilnosHighButton;
	JPanel buttonPanel;

	// Declaring JFileChooser, reader, writer, and file
	JFileChooser fc;
	FileReader fr;
	FileWriter fw;
	File file;

	// Declaring the two crosshairs and an overlay applied to the chart
	Crosshair xCrosshair;
	Crosshair yCrosshair;
	CrosshairOverlay crosshairOverlay;

	// Declaring a CurrentChartModel instance, containing all data
	CurrentChartModel currentModel;

	// Constructor creating the frame GUI
	public CurrentChartView(final String title,
			CurrentChartModel inputCurrentModel) {

		// declaring an instance of the CurrentCharModel from input
		currentModel = inputCurrentModel;

		// creating the chart and the chart panel
		chart = createChart(currentModel.dataset, title);
		chartPanel = new ChartPanel(chart);
		chartPanel.setLayout(null);

		// creating the button panel
		buttonPanel = new JPanel();
		buttonPanel.setLayout(null);

		// creating and overlaying crosshairs to the chart panel
		this.chartPanel.addChartMouseListener(this);
		crosshairOverlay = new CrosshairOverlay();
		this.xCrosshair = new Crosshair(Double.NaN, Color.GRAY,
				new BasicStroke(0f));
		this.xCrosshair.setLabelVisible(true);
		this.yCrosshair = new Crosshair(Double.NaN, Color.GRAY,
				new BasicStroke(0f));
		this.yCrosshair.setLabelVisible(true);
		crosshairOverlay.addDomainCrosshair(xCrosshair);
		crosshairOverlay.addRangeCrosshair(yCrosshair);
		chartPanel.addOverlay(crosshairOverlay);

		// Setting boundaries and adding the chart panel to the frame
		chartPanel.setBounds(0, 0, (int) (Util.XSIZE * 0.95),
				(int) (Util.YSIZE * 0.8));
		add(chartPanel);

		// Creating the buttons, setting boundaries, and adding them to the
		// buttonPanel
		lowPassButton();
		lowPassButton.setBounds(0, (int) (Util.YSIZE * 0.8), 220, 25);
		buttonPanel.add(lowPassButton);
		highPassButton();
		highPassButton.setBounds(230, (int) (Util.YSIZE * 0.8), 220, 25);
		buttonPanel.add(highPassButton);
		restoreButton();
		restoreButton.setBounds(460, (int) (Util.YSIZE * 0.8), 220, 25);
		buttonPanel.add(restoreButton);
		saveButton();
		saveButton.setBounds(690, (int) (Util.YSIZE * 0.8), 220, 25);
		buttonPanel.add(saveButton);
		transient2Button();
		transient2Button.setBounds(920, (int) (Util.YSIZE * 0.8), 220, 25);
		buttonPanel.add(transient2Button);
		transient3Button();
		transient3Button.setBounds(1150, (int) (Util.YSIZE * 0.8), 220, 25);
		buttonPanel.add(transient3Button);
		ficksCurveButton();
		ficksCurveButton.setBounds(0, (int) (Util.YSIZE * 0.8 + 30), 220, 25);
		buttonPanel.add(ficksCurveButton);
		ficksElapsedButton();
		ficksElapsedButton.setBounds(230, (int) (Util.YSIZE * 0.8 + 30), 220,
				25);
		buttonPanel.add(ficksElapsedButton);
		ficksFitButton();
		ficksFitButton.setBounds(460, (int) (Util.YSIZE * 0.8 + 30), 220, 25);
		buttonPanel.add(ficksFitButton);
		ilnosFitButton();
		ilnosFitButton.setBounds(690, (int) (Util.YSIZE * 0.8 + 30), 220, 25);
		buttonPanel.add(ilnosFitButton);
		ilnosLowButton();
		ilnosLowButton.setBounds(920, (int) (Util.YSIZE * 0.8 + 30), 220, 25);
		buttonPanel.add(ilnosLowButton);
		ilnosHighButton();
		ilnosHighButton.setBounds(1150, (int) (Util.YSIZE * 0.8 + 30), 220, 25);
		buttonPanel.add(ilnosHighButton);

		// Setting boundaries for the buttonPanel and adding it to the frame
		buttonPanel.setBounds(0, (int) (Util.YSIZE * 0.8),
				(int) (Util.XSIZE * 0.95), (int) (Util.YSIZE * 0.2));
		add(buttonPanel);

		// Setting the size of the frame and making it visible
		setSize(Util.XSIZE, (int) (Util.YSIZE * 0.95));
		setVisible(true);
		
		// Setting flag to not running when closing window
		close();

	}
	
	// Setting flag to not running when closing window
	public void close(){
		addWindowListener(new WindowAdapter() {
			  public void windowClosing(WindowEvent e) {
			   currentModel.run = false;
			  }
			});
	}
	

	// Creating the chart
	private JFreeChart createChart(final XYDataset dataset, String title) {

		// Initializing the chart
		final JFreeChart chart = ChartFactory.createXYLineChart(
				title, // chart title
				"Time (s)", // x axis label
				"Current (A)", // y axis label
				dataset, // data
				PlotOrientation.VERTICAL, true, // include legend
				true, // tooltips
				false // urls
				);

		// Customization of the chart
		chart.setBackgroundPaint(Color.white);

		// Make a reference to the plot for further customization
		final XYPlot plot = chart.getXYPlot();
		plot.setBackgroundPaint(Color.lightGray);

		// Set color of grid
		plot.setDomainGridlinePaint(Color.white);
		plot.setRangeGridlinePaint(Color.white);

		// Rendering the line representation, removing point representations
		// from the series and returning the chart
		final XYLineAndShapeRenderer renderer = new XYLineAndShapeRenderer();
		renderer.setSeriesLinesVisible(0, true);
		renderer.setSeriesShapesVisible(0, false);
		renderer.setSeriesShapesVisible(1, false);
		renderer.setSeriesShapesVisible(2, false);
		renderer.setSeriesShapesVisible(3, false);
		renderer.setSeriesShapesVisible(4, false);
		renderer.setSeriesShapesVisible(5, false);
		renderer.setSeriesShapesVisible(6, false);
		renderer.setSeriesShapesVisible(7, false);
		plot.setRenderer(renderer);
		return chart;

	}

	// Remove points in chart if doubleclicked
		@Override
		public void chartMouseClicked(ChartMouseEvent event) {
			MouseEvent me = event.getTrigger();
		    if ( me.getModifiers() == 16 ) { //Left Button ?
		        if ( me.getClickCount() == 2) { //Double Click?
		            me.consume();
		            XYItemEntity entity = (XYItemEntity) event.getEntity();
		            int item = entity.getItem();
		            currentModel.removePoint(item);
		            System.out.println("entity: " + entity + " item: " + item);
		        }
		    }
		}

	// Overriding the mouse movement on the chart, making crosshairs visible
	@Override
	public void chartMouseMoved(ChartMouseEvent event) {
		Rectangle2D dataArea = this.chartPanel.getScreenDataArea();
		JFreeChart chart = event.getChart();
		XYPlot plot = (XYPlot) chart.getPlot();
		ValueAxis xAxis = plot.getDomainAxis();
		double x = xAxis.java2DToValue(event.getTrigger().getX(), dataArea,
				RectangleEdge.BOTTOM);
		double y = DatasetUtilities.findYValue(plot.getDataset(), 0, x);
		this.xCrosshair.setValue(x);
		this.yCrosshair.setValue(y);

	}

	// Create button for low pass filter
	public void lowPassButton() {
		
		// Initializing button
		lowPassButton = new JButton("Low Pass Filter");

		// When low pass button is pressed
		lowPassButton.addActionListener(new ActionListener() {

			public void actionPerformed(ActionEvent e) {
				currentModel.lowPassFilter();
			}
		});
	}

	// Create button for low pass filter
	public void highPassButton() {
		
		// Initializing button
		highPassButton = new JButton("High Pass Filter");

		// When low pass button is pressed
		highPassButton.addActionListener(new ActionListener() {

			public void actionPerformed(ActionEvent e) {
				currentModel.highPassFilter();
			}
		});
	}

	// Create button to restore original
	public void restoreButton() {
		
		// Initializing button
		restoreButton = new JButton("Restore");

		// When restore button is pressed
		restoreButton.addActionListener(new ActionListener() {

			public void actionPerformed(ActionEvent e) {
				currentModel.restore();
			}
		});
	}

	// Create button for transient 2
	public void transient2Button() {

		// Initializing button
		transient2Button = new JButton("Transient 2");

		// When restore button is pressed
		transient2Button.addActionListener(new ActionListener() {

			public void actionPerformed(ActionEvent e) {
				try {
					// Initializing JFileChooser
					fc = new JFileChooser(System.getProperty("user.home")
							+ File.separator + "Desktop" + File.separator);

					// Reading file
					if (fc.showOpenDialog(null) == JFileChooser.APPROVE_OPTION) {
						file = fc.getSelectedFile();
						fr = new FileReader(file);
						BufferedReader br = new BufferedReader(fr);
						currentModel.transient2(br);
					}
				} catch (IOException e1) {
					// TODO Auto-generated catch block
					e1.printStackTrace();
				}
			}
		});
	}

	// Create button for transient 3
	public void transient3Button() {

		// Initializing button
		transient3Button = new JButton("Transient 3");

		// When restore button is pressed
		transient3Button.addActionListener(new ActionListener() {

			public void actionPerformed(ActionEvent e) {
				try {
					// Initializing JFileChooser
					fc = new JFileChooser(System.getProperty("user.home")
							+ File.separator + "Desktop" + File.separator);

					// Reading file
					if (fc.showOpenDialog(null) == JFileChooser.APPROVE_OPTION) {
						file = fc.getSelectedFile();
						fr = new FileReader(file);
						BufferedReader br = new BufferedReader(fr);
						currentModel.transient3(br);
					}
				} catch (IOException e1) {
					// TODO Auto-generated catch block
					e1.printStackTrace();
				}
			}
		});
	}

	// Create button for saving
	public void saveButton() {

		// Initializing button
		saveButton = new JButton("Save");

		// When restore button is pressed
		saveButton.addActionListener(new ActionListener() {

			public void actionPerformed(ActionEvent e) {
				// Initializing JFileChooser
				fc = new JFileChooser(System.getProperty("user.home")
						+ File.separator + "Desktop" + File.separator);

				// Initializing the file writer, and writing the series to a
				// .txt file
				fw = null;
				if (fc.showSaveDialog(null) == JFileChooser.APPROVE_OPTION) {
					try {
						fw = new FileWriter(fc.getSelectedFile() + ".txt");
						currentModel.save(fw);
						fw.close();

					} catch (IOException e1) {
						// TODO Auto-generated catch block
						e1.printStackTrace();
					}
				}
			}
		});
	}

	// Create button to create Fick's lattice curve
	public void ficksCurveButton() {
		
		// Initializing button
		ficksCurveButton = new JButton("Fick's Lattice Curve");

		// When Fick's curve fitting button is pressed
		ficksCurveButton.addActionListener(new ActionListener() {

			public void actionPerformed(ActionEvent e) {
				currentModel.ficksCurve();
			}
		});
	}

	// Create button to create Fick's elapsed time curve
	public void ficksElapsedButton() {
		
		// Initializing button
		ficksElapsedButton = new JButton("Fick's Elapsed Time Curve");

		// When Fick's curve fitting button is pressed
		ficksElapsedButton.addActionListener(new ActionListener() {

			public void actionPerformed(ActionEvent e) {
				currentModel.ficksElapsed();
			}
		});
	}

	// Create button to create Fick's numerical fit curve
	public void ficksFitButton() {
		
		// Initializing button
		ficksFitButton = new JButton("Fick's Curve Fitting");

		// When Fick's curve fitting button is pressed
		ficksFitButton.addActionListener(new ActionListener() {

			public void actionPerformed(ActionEvent e) {
				currentModel.ficksFit();
			}
		});
	}

	// Create button to create Ilno's numerical fit curve
	public void ilnosFitButton() {
		
		// Initializing button
		ilnosFitButton = new JButton("Ilno's Curve Fitting");

		// When Ilno's curve fitting button is pressed
		ilnosFitButton.addActionListener(new ActionListener() {

			public void actionPerformed(ActionEvent e) {
				long start = System.currentTimeMillis();
				currentModel.ilnosConditions();
				System.out.println("Time: "
						+ (System.currentTimeMillis() - start) / 60000);
			}

		});
	}

	// Create button to apply Ilno's low pass filter
	public void ilnosLowButton() {
		
		// Initializing button
		ilnosLowButton = new JButton("Ilno's Low Pass");

		// When Ilno's low pass button is pressed
		ilnosLowButton.addActionListener(new ActionListener() {

			public void actionPerformed(ActionEvent e) {
				currentModel.ilnosLow();
			}

		});
	}

	// Create button to apply Ilno's high pass filter
	public void ilnosHighButton() {
		// Initializing button
		ilnosHighButton = new JButton("Ilno's High Pass");

		// When Ilno's high pass button is pressed
		ilnosHighButton.addActionListener(new ActionListener() {

			public void actionPerformed(ActionEvent e) {
				currentModel.ilnosHigh();
			}

		});
	}
	
}