package diffusioncalculator;

import java.io.BufferedReader;
import java.io.FileWriter;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Collections;

import org.jfree.data.xy.XYSeries;
import org.jfree.data.xy.XYSeriesCollection;

/* Uses the JFREECHART Library to make the data set plot in the Current/Time chart. 
 * Also implements the low pass filter, and the high pass filter which removes data 
 * over/under a certain threshold. Lastly, it implements the restore button
 * which removes data in the chart, and loads the original content*/
public class CurrentChartModel {

	// Declaring lists containing data to be plotted
	public ArrayList<Float> time;
	public ArrayList<Float> current;

	// Declaring JFREECHART elements
	XYSeriesCollection dataset;
	XYSeries series1;
	XYSeries series2;
	XYSeries series3;
	XYSeries ficksCurve;

	// Read and write variables
	String line;
	String[] values;

	// Declaring locally used scale and temporary variables
	double scaleCurrent;
	float temp;
	double fick;
	float L;
	float deff;

	// Constructor initializing the data set used by the chart, taking data
	// arrays as inputs
	public CurrentChartModel(ArrayList<Float> inputTime,
			ArrayList<Float> inputCurrent, float inputL, float inputDeff) throws IOException {
		deff = inputDeff;
		L = inputL;
		time = inputTime;
		current = inputCurrent;
		series1 = new XYSeries("Transient 1");
		series2 = new XYSeries("Transient 2");
		series3 = new XYSeries("Transient 3");
		ficksCurve = new XYSeries("Fick's Curve");

		// Adding x and y values to the series using the restore
		// function
		restore();

		// Adding the series to the data set
		dataset = new XYSeriesCollection();
		dataset.addSeries(series1);
		dataset.addSeries(ficksCurve);
	}

	// Implementing the low pass filter method
	public void lowPassFilter() {

		// Declaring holder variables for the last y value, and the one before
		// there again
		float lastValue = 0f;
		float lastlastValue = 0f;

		// Declaring a counter and a holder to help parse the y value
		int counter = 0;
		double db;

		// Looking for abnormal values and replacing them with the last y value
		for (int i = 0; i < time.size(); i++) {
			db = (double) (series1.getY(i));
			temp = (float) db;
			if (temp - lastValue > 0.02962) {
				series1.remove(i);
				series1.add(series1.getX(i - 1), (double) (lastValue));
			} else if (temp - lastlastValue > 0.04) {
				series1.remove(i);
				series1.add(series1.getX(i - 1), (double) (lastValue));
			} else {
				lastValue = temp;
				counter++;
			}
			if (counter == 2) {
				lastlastValue = lastValue;
				counter = 0;
			}

		}
	}

	// Implementing the high pass filter method
	public void highPassFilter() {

		// Declaring holder variables for the last y value, and the one before
		// there again
		float lastValue = 0f;
		float lastlastValue = 0f;

		// Declaring a counter and a holder to help parse the y value
		int counter = 0;
		double db;

		// Looking for abnormal values and replacing them with the last y value
		for (int i = 0; i < time.size(); i++) {
			db = (double) (series1.getY(i));
			temp = (float) db;
			if (temp - lastValue < 0.02962) {
				series1.remove(i);
				series1.add(series1.getX(i - 1), (double) (lastValue));
			} else if (temp - lastlastValue < 0.04) {
				series1.remove(i);
				series1.add(series1.getX(i - 1), (double) (lastValue));
			} else {
				lastValue = temp;
				counter++;
			}
			if (counter == 2) {
				lastlastValue = lastValue;
				counter = 0;
			}

		}
	}

	// Restoring the original plot
	public void restore() {

		// Removing existing data and inputting new
		series1.clear();
		series2.clear();
		series3.clear();
		ficksCurve.clear();
		for (int i = 0; i < time.size(); i++) {

			// Scaling values to micro
			scaleCurrent = current.get(i) * 1000000;
			series1.add((double) (time.get(i)), scaleCurrent);
		}
		// Adding fick's curve data
				temp = 0;
				for (double i = 1; i < time.size(); i++){
					fick =  (1 + 2 * (Math.exp(-36 * Math.PI * Math.PI * ((3*i*deff)/(L*L)))
							- Math.exp(-25 * Math.PI * Math.PI * ((3*i*deff)/(L*L)))
							+ Math.exp(-16 * Math.PI * Math.PI * ((3*i*deff)/(L*L)))
							- Math.exp(-9 * Math.PI * Math.PI * ((3*i*deff)/(L*L)))
							+ Math.exp(-4 * Math.PI * Math.PI * ((3*i*deff)/(L*L))) - Math.exp(-Math.PI
							* Math.PI * ((3*i*deff)/(L*L))))) *Collections.max(current)*1000000;
					temp = (time.get((int) i));
					ficksCurve.add(temp, fick);
				}
	}

	// Implementing writer method for transient 2
	public void transient2(BufferedReader br) throws IOException {
		series2.clear();
		dataset.removeSeries(series2);
		while ((line = br.readLine()) != null) {
			values = line.split("\t");
			series2.add(Double.parseDouble(values[0]),
					Double.parseDouble(values[1]));
		}
		dataset.addSeries(series2);
		br.close();
	}
	
	// Implementing writer method for transient 3
		public void transient3(BufferedReader br) throws IOException {
			series3.clear();
			dataset.removeSeries(series3);
			while ((line = br.readLine()) != null) {
				values = line.split("\t");
				series3.add(Double.parseDouble(values[0]),
						Double.parseDouble(values[1]));
			}
			dataset.addSeries(series3);
			br.close();
		}

	// Implementing a method to save series data locally, to be used later
	public void save(FileWriter fw) throws IOException {

		for (int i = 0; i < time.size(); i++) {
			fw.write(series1.getX(i) + "\t" + series1.getY(i)
					+ System.getProperty("line.separator"));
		}
	}
}
